@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2019 by Sukchan Lee <acetcom@gmail.com>
+ * Copyright (C) 2022 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  *
  * This file is part of Open5GS.
  *
@@ -36,38 +37,89 @@ ogs_tlv_desc_t ogs_tlv_desc_more7 = {
 ogs_tlv_desc_t ogs_tlv_desc_more8 = {
     OGS_TLV_MORE, "More", 0, 8, 0, 0, { NULL } };
 
-static ogs_tlv_t *tlv_add_leaf(
-        ogs_tlv_t *parent_tlv, ogs_tlv_t *tlv, ogs_tlv_desc_t *desc, void *msg)
+/* Return specific TLV mode based on its TLV description type and the msg
+ * provided mode (used to know the type length) */
+static uint8_t tlv_ctype2mode(ogs_tlv_type_e ctype, uint8_t msg_mode)
 {
+    switch(ctype) {
+    case OGS_TLV_UINT8:
+    case OGS_TLV_UINT16:
+    case OGS_TLV_UINT24:
+    case OGS_TLV_UINT32:
+    case OGS_TLV_INT8:
+    case OGS_TLV_INT16:
+    case OGS_TLV_INT24:
+    case OGS_TLV_INT32:
+    case OGS_TLV_FIXED_STR:
+    case OGS_TLV_VAR_STR:
+    case OGS_TLV_NULL:
+    case OGS_TLV_MORE:
+    case OGS_TLV_COMPOUND:
+    case OGS_TLV_MESSAGE:
+        return msg_mode;
+    case OGS_TV_UINT8:
+    case OGS_TV_UINT16:
+    case OGS_TV_UINT24:
+    case OGS_TV_UINT32:
+    case OGS_TV_INT8:
+    case OGS_TV_INT16:
+    case OGS_TV_INT24:
+    case OGS_TV_INT32:
+    case OGS_TV_FIXED_STR:
+    case OGS_TV_VAR_STR:
+    case OGS_TV_NULL:
+    case OGS_TV_MORE:
+        if (msg_mode == OGS_TLV_MODE_T2_L2)
+            return OGS_TLV_MODE_T1; /* Here return OGS_TLV_MODE_T2 if ever existing */
+        else
+            return OGS_TLV_MODE_T1;
+    default:
+        ogs_assert_if_reached();
+        break;
+    }
+}
+
+static ogs_tlv_t *tlv_add_leaf(
+        ogs_tlv_t *parent_tlv, ogs_tlv_t *tlv, ogs_tlv_desc_t *desc, void *msg, uint8_t msg_mode)
+{
+    uint8_t tlv_mode = tlv_ctype2mode(desc->ctype, msg_mode);
+
     switch (desc->ctype) {
     case OGS_TLV_UINT8:
     case OGS_TLV_INT8:
+    case OGS_TV_UINT8:
+    case OGS_TV_INT8:
     {
         ogs_tlv_uint8_t *v = (ogs_tlv_uint8_t *)msg;
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, 1, desc->instance, &v->u8);
         else
-            tlv = ogs_tlv_add(tlv, desc->type, 1, desc->instance, &v->u8);
+            tlv = ogs_tlv_add(tlv, tlv_mode, desc->type, 1, desc->instance, &v->u8);
         ogs_expect_or_return_val(tlv, NULL);
         break;
     }
     case OGS_TLV_UINT16:
+    case OGS_TLV_INT16:
+    case OGS_TV_UINT16:
+    case OGS_TV_INT16:
     {
         ogs_tlv_uint16_t *v = (ogs_tlv_uint16_t *)msg;
 
         v->u16 = htobe16(v->u16);
 
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, 2, desc->instance, &v->u16);
         else
-            tlv = ogs_tlv_add(tlv, desc->type, 2, desc->instance, &v->u16);
+            tlv = ogs_tlv_add(tlv, tlv_mode, desc->type, 2, desc->instance, &v->u16);
         ogs_expect_or_return_val(tlv, NULL);
         break;
     }
     case OGS_TLV_UINT24:
     case OGS_TLV_INT24:
+    case OGS_TV_UINT24:
+    case OGS_TV_INT24:
     {
         ogs_tlv_uint24_t *v = (ogs_tlv_uint24_t *)msg;
 
@@ -75,38 +127,41 @@ static ogs_tlv_t *tlv_add_leaf(
         v->u24 = htobe32(v->u24);
 
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, 3, desc->instance, &v->u24);
         else
-            tlv = ogs_tlv_add(tlv, desc->type, 3, desc->instance, &v->u24);
+            tlv = ogs_tlv_add(tlv, tlv_mode, desc->type, 3, desc->instance, &v->u24);
         ogs_expect_or_return_val(tlv, NULL);
         break;
     }
     case OGS_TLV_UINT32:
     case OGS_TLV_INT32:
+    case OGS_TV_UINT32:
+    case OGS_TV_INT32:
     {
         ogs_tlv_uint32_t *v = (ogs_tlv_uint32_t *)msg;
 
         v->u32 = htobe32(v->u32);
 
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, 4, desc->instance, &v->u32);
         else
-            tlv = ogs_tlv_add(tlv,
+            tlv = ogs_tlv_add(tlv, tlv_mode,
                     desc->type, 4, desc->instance, &v->u32);
         ogs_expect_or_return_val(tlv, NULL);
         break;
     }
     case OGS_TLV_FIXED_STR:
+    case OGS_TV_FIXED_STR:
     {
         ogs_tlv_octet_t *v = (ogs_tlv_octet_t *)msg;
 
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, desc->length, desc->instance, v->data);
         else
-            tlv = ogs_tlv_add(tlv,
+            tlv = ogs_tlv_add(tlv, tlv_mode,
                     desc->type, desc->length, desc->instance, v->data);
         ogs_expect_or_return_val(tlv, NULL);
         break;
@@ -122,21 +177,22 @@ static ogs_tlv_t *tlv_add_leaf(
         }
 
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, v->len, desc->instance, v->data);
         else
-            tlv = ogs_tlv_add(tlv,
+            tlv = ogs_tlv_add(tlv, tlv_mode,
                     desc->type, v->len, desc->instance, v->data);
         ogs_expect_or_return_val(tlv, NULL);
         break;
     }
     case OGS_TLV_NULL:
+    case OGS_TV_NULL:
     {
         if (parent_tlv)
-            tlv = ogs_tlv_embed(parent_tlv,
+            tlv = ogs_tlv_embed(parent_tlv, tlv_mode,
                     desc->type, 0, desc->instance, NULL);
         else
-            tlv = ogs_tlv_add(tlv,
+            tlv = ogs_tlv_add(tlv, tlv_mode,
                     desc->type, 0, desc->instance, NULL);
         ogs_expect_or_return_val(tlv, NULL);
         break;
@@ -150,7 +206,7 @@ static ogs_tlv_t *tlv_add_leaf(
 }
 
 static uint32_t tlv_add_compound(ogs_tlv_t **root, ogs_tlv_t *parent_tlv,
-        ogs_tlv_desc_t *parent_desc, void *msg, int depth)
+        ogs_tlv_desc_t *parent_desc, void *msg, int depth, uint8_t mode)
 {
     ogs_tlv_presence_t *presence_p;
     ogs_tlv_desc_t *desc = NULL, *next_desc = NULL;
@@ -186,15 +242,15 @@ static uint32_t tlv_add_compound(ogs_tlv_t **root, ogs_tlv_t *parent_tlv,
                             desc->vsize, p + offset2);
 
                     if (parent_tlv)
-                        tlv = ogs_tlv_embed(parent_tlv,
+                        tlv = ogs_tlv_embed(parent_tlv, tlv_ctype2mode(desc->ctype, mode),
                                 desc->type, 0, desc->instance, NULL);
                     else
-                        tlv = ogs_tlv_add(tlv,
+                        tlv = ogs_tlv_add(tlv, tlv_ctype2mode(desc->ctype, mode),
                                 desc->type, 0, desc->instance, NULL);
 
                     r = tlv_add_compound(&emb_tlv, tlv, desc,
                             p + offset2 + sizeof(ogs_tlv_presence_t),
-                            depth + 1);
+                            depth + 1, mode);
                     ogs_expect_or_return_val(r > 0 && emb_tlv, 0);
                     count += 1 + r;
                 } else {
@@ -204,7 +260,7 @@ static uint32_t tlv_add_compound(ogs_tlv_t **root, ogs_tlv_t *parent_tlv,
                             desc->instance, desc->ctype, desc->vsize,
                             p + offset2);
 
-                    tlv = tlv_add_leaf(parent_tlv, tlv, desc, p + offset2);
+                    tlv = tlv_add_leaf(parent_tlv, tlv, desc, p + offset2, mode);
                     ogs_expect_or_return_val(tlv, 0);
                     count++;
                 }
@@ -226,15 +282,15 @@ static uint32_t tlv_add_compound(ogs_tlv_t **root, ogs_tlv_t *parent_tlv,
                             desc->vsize, p + offset);
 
                     if (parent_tlv)
-                        tlv = ogs_tlv_embed(parent_tlv,
+                        tlv = ogs_tlv_embed(parent_tlv, tlv_ctype2mode(desc->ctype, mode),
                                 desc->type, 0, desc->instance, NULL);
                     else
-                        tlv = ogs_tlv_add(tlv,
+                        tlv = ogs_tlv_add(tlv, tlv_ctype2mode(desc->ctype, mode),
                                 desc->type, 0, desc->instance, NULL);
 
                     r = tlv_add_compound(&emb_tlv, tlv, desc,
                             p + offset + sizeof(ogs_tlv_presence_t),
-                            depth + 1);
+                            depth + 1, mode);
                     ogs_expect_or_return_val(r > 0 && emb_tlv, 0);
                     count += 1 + r;
                 } else {
@@ -244,7 +300,7 @@ static uint32_t tlv_add_compound(ogs_tlv_t **root, ogs_tlv_t *parent_tlv,
                             desc->instance, desc->ctype, desc->vsize,
                             p + offset);
 
-                    tlv = tlv_add_leaf(parent_tlv, tlv, desc, p + offset);
+                    tlv = tlv_add_leaf(parent_tlv, tlv, desc, p + offset, mode);
                     ogs_expect_or_return_val(tlv, 0);
                     count++;
                 }
@@ -271,10 +327,10 @@ ogs_pkbuf_t *ogs_tlv_build_msg(ogs_tlv_desc_t *desc, void *msg, int mode)
     ogs_assert(desc->ctype == OGS_TLV_MESSAGE);
 
     if (desc->child_descs[0]) {
-        r = tlv_add_compound(&root, NULL, desc, msg, 0);
+        r = tlv_add_compound(&root, NULL, desc, msg, 0, mode);
         ogs_expect_or_return_val(r > 0 && root, NULL);
 
-        length = ogs_tlv_calc_length(root, mode);
+        length = ogs_tlv_calc_length(root);
     } else {
         length = 0;
     }
@@ -284,7 +340,7 @@ ogs_pkbuf_t *ogs_tlv_build_msg(ogs_tlv_desc_t *desc, void *msg, int mode)
     ogs_pkbuf_put(pkbuf, length);
 
     if (desc->child_descs[0]) {
-        rendlen = ogs_tlv_render(root, pkbuf->data, length, mode);
+        rendlen = ogs_tlv_render(root, pkbuf->data, length);
         ogs_expect_or_return_val(rendlen == length, NULL);
 
         ogs_tlv_free_all(root);
@@ -293,21 +349,24 @@ ogs_pkbuf_t *ogs_tlv_build_msg(ogs_tlv_desc_t *desc, void *msg, int mode)
     return pkbuf;
 }
 
-static ogs_tlv_desc_t* tlv_find_desc(uint8_t *desc_index,
-        uint32_t *tlv_offset, ogs_tlv_desc_t *parent_desc, ogs_tlv_t *tlv)
+static ogs_tlv_desc_t* tlv_find_desc_by_type_inst(uint8_t *desc_index,
+        uint32_t *tlv_offset, ogs_tlv_desc_t *parent_desc, uint16_t match_type, uint8_t match_instance, uint8_t match_type_pos)
 {
     ogs_tlv_desc_t *prev_desc = NULL, *desc = NULL;
     int i, offset = 0;
+    unsigned match_i = 0;
 
     ogs_assert(parent_desc);
-    ogs_assert(tlv);
 
     for (i = 0, desc = parent_desc->child_descs[i]; desc != NULL;
             i++, desc = parent_desc->child_descs[i]) {
-        if (desc->type == tlv->type && desc->instance == tlv->instance) {
-            *desc_index = i;
-            *tlv_offset = offset;
-            break;
+        if (desc->type == match_type && desc->instance == match_instance) {
+            if (match_i == match_type_pos) {
+                *desc_index = i;
+                *tlv_offset = offset;
+                break;
+            }
+            match_i++;
         }
 
         if (desc->ctype == OGS_TLV_MORE) {
@@ -330,6 +389,8 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
     ogs_assert(tlv);
 
     switch (desc->ctype) {
+    case OGS_TV_UINT8:
+    case OGS_TV_INT8:
     case OGS_TLV_UINT8:
     case OGS_TLV_INT8:
     {
@@ -343,6 +404,8 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
         v->u8 = *(uint8_t*)(tlv->value);
         break;
     }
+    case OGS_TV_UINT16:
+    case OGS_TV_INT16:
     case OGS_TLV_UINT16:
     case OGS_TLV_INT16:
     {
@@ -357,6 +420,8 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
                ((((uint8_t*)tlv->value)[1]    )&0x00ff);
         break;
     }
+    case OGS_TV_UINT24:
+    case OGS_TV_INT24:
     case OGS_TLV_UINT24:
     case OGS_TLV_INT24:
     {
@@ -372,6 +437,8 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
                ((((uint8_t*)tlv->value)[2]    )&0x000000ff);
         break;
     }
+    case OGS_TV_UINT32:
+    case OGS_TV_INT32:
     case OGS_TLV_UINT32:
     case OGS_TLV_INT32:
     {
@@ -388,6 +455,7 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
                ((((uint8_t*)tlv->value)[3]    )&0x000000ff);
         break;
     }
+    case OGS_TV_FIXED_STR:
     case OGS_TLV_FIXED_STR:
     {
         ogs_tlv_octet_t *v = (ogs_tlv_octet_t *)msg;
@@ -411,6 +479,7 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
         v->len = tlv->length;
         break;
     }
+    case OGS_TV_NULL:
     case OGS_TLV_NULL:
     {
         if (tlv->length != 0) {
@@ -427,6 +496,50 @@ static int tlv_parse_leaf(void *msg, ogs_tlv_desc_t *desc, ogs_tlv_t *tlv)
     return OGS_OK;
 }
 
+/* rbtree structure used to keep track count of TLVs with given <type,instance>
+ * while parsing. This is used to link it to the matching nth field in a
+ * ogs_tlv_desc_t struct */
+typedef struct tlv_count_node_s {
+    ogs_rbnode_t node;
+    uint32_t key; /* type<<8 + instance */
+    unsigned count;
+} tlv_count_node_t;
+
+static tlv_count_node_t *tlv_count_node_find(ogs_rbtree_t *tree, tlv_count_node_t *count_node_arr, unsigned count_node_arr_len,
+        unsigned *count_alloc_next, uint16_t type, uint8_t instance)
+{
+    ogs_rbnode_t **new = NULL;
+    ogs_rbnode_t *parent = NULL;
+    tlv_count_node_t *this;
+
+    uint64_t key = (((uint32_t)type)<<8) | instance;
+
+    new = &tree->root;
+    while (*new) {
+        this = ogs_rb_entry(*new, tlv_count_node_t, node);
+        parent = *new;
+        if (key < this->key) {
+            new = &(*new)->left;
+        } else if (key > this->key) {
+            new = &(*new)->right;
+        } else { /* found entry: */
+            return this;
+        }
+    }
+
+    /* No entry, need to add one: */
+    if (*count_alloc_next == count_node_arr_len) {
+        ogs_error("This TLV has to many entries, can't parse");
+        return NULL;
+    }
+    this = &count_node_arr[(*count_alloc_next)++];
+    this->key = key;
+    this->count = 0;
+    ogs_rbtree_link_node(&this->node, parent, new);
+    ogs_rbtree_insert_color(tree, &this->node);
+    return this;
+}
+
 static int tlv_parse_compound(void *msg, ogs_tlv_desc_t *parent_desc,
         ogs_tlv_t *parent_tlv, int depth, int mode)
 {
@@ -438,6 +551,9 @@ static int tlv_parse_compound(void *msg, ogs_tlv_desc_t *parent_desc,
     uint32_t offset = 0;
     uint8_t index = 0;
     int i = 0, j;
+    OGS_RBTREE(tree);
+    tlv_count_node_t count_node[OGS_TLV_MAX_CHILD_DESC];
+    unsigned count_node_alloc_next = 0;
     char indent[17] = "                "; /* 16 spaces */
 
     ogs_assert(msg);
@@ -449,7 +565,11 @@ static int tlv_parse_compound(void *msg, ogs_tlv_desc_t *parent_desc,
 
     tlv = parent_tlv;
     while (tlv) {
-        desc = tlv_find_desc(&index, &offset, parent_desc, tlv);
+        tlv_count_node_t *curr_count = tlv_count_node_find(&tree, &count_node[0],
+                OGS_ARRAY_SIZE(count_node), &count_node_alloc_next, tlv->type, tlv->instance);
+        if (!curr_count)
+            return OGS_ERROR;
+        desc = tlv_find_desc_by_type_inst(&index, &offset, parent_desc, tlv->type, tlv->instance, curr_count->count);
         if (desc == NULL) {
             ogs_warn("Unknown TLV type [%d]", tlv->type);
             tlv = tlv->next;
@@ -475,6 +595,8 @@ static int tlv_parse_compound(void *msg, ogs_tlv_desc_t *parent_desc,
                 tlv = tlv->next;
                 continue;
             }
+        } else {
+            curr_count->count++;
         }
 
         if (desc->ctype == OGS_TLV_COMPOUND) {
@@ -544,3 +666,117 @@ int ogs_tlv_parse_msg(void *msg, ogs_tlv_desc_t *desc, ogs_pkbuf_t *pkbuf,
 
     return rv;
 }
+
+static uint16_t parse_get_element_type(uint8_t *pos, uint8_t mode)
+{
+    uint16_t type;
+
+    switch(mode) {
+    case OGS_TLV_MODE_T1_L1:
+    case OGS_TLV_MODE_T1_L2:
+    case OGS_TLV_MODE_T1_L2_I1:
+    case OGS_TLV_MODE_T1:
+        type = *pos;
+        break;
+    case OGS_TLV_MODE_T2_L2:
+        type = *(pos++) << 8;
+        type += *(pos++);
+        break;
+    default:
+        ogs_assert_if_reached();
+        break;
+    }
+
+    return type;
+}
+
+/* Get TLV element taking into account msg_mode (to know TLV tag length) +
+ * specific TLV information from "desc" (to know whether the specific IE is TLV
+ * or TV, and its expected length in the later case). */
+static uint8_t *tlv_get_element_desc(ogs_tlv_t *tlv, uint8_t *blk, uint8_t msg_mode, ogs_tlv_desc_t *desc)
+{
+    uint8_t instance;
+    unsigned tlv_tag_pos;
+    uint8_t desc_index = 0;
+    uint32_t tlv_offset = 0;
+    uint16_t tlv_tag;
+    uint8_t tlv_mode;
+    static ogs_tlv_desc_t* tlv_desc;
+
+    tlv_tag = parse_get_element_type(blk, msg_mode);
+    instance = 0;  /* TODO: support instance != 0 if ever really needed by looking it up in pos */
+    tlv_tag_pos = 0; /* All tags with same instance should use the same tlv_desc, so take the first one */
+    tlv_desc = tlv_find_desc_by_type_inst(&desc_index, &tlv_offset, desc, tlv_tag, instance, tlv_tag_pos);
+    if (!tlv_desc) {
+        ogs_error("Can't parse find TLV description for type %u", tlv_tag);
+        return NULL;
+    }
+    tlv_mode = tlv_ctype2mode(tlv_desc->ctype, msg_mode);
+
+    if (tlv_mode == OGS_TLV_MODE_T1)
+        return tlv_get_element_fixed(tlv, blk, tlv_mode, tlv_desc->length);
+    return tlv_get_element(tlv, blk, tlv_mode);
+}
+
+/* Similar to ogs_tlv_parse_block(), but taking into account each TLV format from "desc". */
+static ogs_tlv_t *ogs_tlv_parse_block_desc(uint32_t length, void *data, uint8_t msg_mode, ogs_tlv_desc_t *desc)
+{
+    uint8_t *pos = data;
+    uint8_t *blk = data;
+
+    ogs_tlv_t *root = NULL;
+    ogs_tlv_t *prev = NULL;
+    ogs_tlv_t *curr = NULL;
+
+    root = curr = ogs_tlv_get();
+
+    ogs_assert(curr);
+
+    pos = tlv_get_element_desc(curr, pos, msg_mode, desc);
+
+    ogs_assert(pos);
+
+    while(pos - blk < length) {
+        prev = curr;
+
+        curr = ogs_tlv_get();
+        ogs_assert(curr);
+        prev->next = curr;
+
+        pos = tlv_get_element_desc(curr, pos, msg_mode, desc);
+        ogs_assert(pos);
+    }
+
+    ogs_assert(length == (pos - blk));
+
+    return root;
+}
+
+/* Similar to ogs_tlv_parse_msg(), but takes each TLV type from the desc
+ * defintion. This allows parsing messages which have different types of TLVs in
+ * it (for instance GTPv1-C). */
+int ogs_tlv_parse_msg_desc(
+        void *msg, ogs_tlv_desc_t *desc, ogs_pkbuf_t *pkbuf, int msg_mode)
+{
+    int rv;
+    ogs_tlv_t *root;
+
+    ogs_assert(msg);
+    ogs_assert(desc);
+    ogs_assert(pkbuf);
+
+    ogs_assert(desc->ctype == OGS_TLV_MESSAGE);
+    ogs_assert(desc->child_descs[0]);
+
+    root = ogs_tlv_parse_block_desc(pkbuf->len, pkbuf->data, msg_mode, desc);
+    if (root == NULL) {
+        ogs_error("Can't parse TLV message");
+        return OGS_ERROR;
+    }
+
+    rv = tlv_parse_compound(msg, desc, root, 0, msg_mode);
+
+    ogs_tlv_free_all(root);
+
+    return rv;
+}
