@@ -105,6 +105,9 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                         const char *dev = NULL;
                         ogs_sockaddr_t *addr = NULL;
 
+                        ogs_sockopt_t option;
+                        bool is_option = false;
+
                         if (ogs_yaml_iter_type(&gtpc_array) ==
                                 YAML_MAPPING_NODE) {
                             memcpy(&gtpc_iter, &gtpc_array,
@@ -161,6 +164,15 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                                 if (v) port = atoi(v);
                             } else if (!strcmp(gtpc_key, "dev")) {
                                 dev = ogs_yaml_iter_value(&gtpc_iter);
+                            } else if (!strcmp(gtpc_key, "option")) {
+                                rv = ogs_app_config_parse_sockopt(
+                                        &gtpc_iter, &option);
+                                if (rv != OGS_OK) return rv;
+                                is_option = true;
+                            } else if (!strcmp(gtpc_key, "tac")) {
+                                /* Nothing */
+                            } else if (!strcmp(gtpc_key, "e_cell_id")) {
+                                /* Nothing */
                             } else
                                 ogs_warn("unknown key `%s`", gtpc_key);
                         }
@@ -175,10 +187,12 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                         if (addr) {
                             if (ogs_app()->parameter.no_ipv4 == 0)
                                 ogs_socknode_add(
-                                        &self.gtpc_list, AF_INET, addr);
+                                    &self.gtpc_list, AF_INET, addr,
+                                    is_option ? &option : NULL);
                             if (ogs_app()->parameter.no_ipv6 == 0)
                                 ogs_socknode_add(
-                                        &self.gtpc_list6, AF_INET6, addr);
+                                    &self.gtpc_list6, AF_INET6, addr,
+                                    is_option ? &option : NULL);
                             ogs_freeaddrinfo(addr);
                         }
 
@@ -188,7 +202,8 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                                         NULL : &self.gtpc_list,
                                     ogs_app()->parameter.no_ipv6 ?
                                         NULL : &self.gtpc_list6,
-                                    dev, port);
+                                    dev, port,
+                                    is_option ? &option : NULL);
                             ogs_assert(rv == OGS_OK);
                         }
 
@@ -202,7 +217,7 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                                     NULL : &self.gtpc_list,
                                 ogs_app()->parameter.no_ipv6 ?
                                     NULL : &self.gtpc_list6,
-                                NULL, self.gtpc_port);
+                                NULL, self.gtpc_port, NULL);
                         ogs_assert(rv == OGS_OK);
                     }
                 } else if (!strcmp(local_key, "gtpu")) {
@@ -229,6 +244,9 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                         const char *network_instance = NULL;
                         const char *source_interface = NULL;
 
+                        ogs_sockopt_t option;
+                        bool is_option = false;
+
                         if (ogs_yaml_iter_type(&gtpu_array) ==
                                 YAML_MAPPING_NODE) {
                             memcpy(&gtpu_iter, &gtpu_array,
@@ -317,6 +335,11 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                                 if (v) port = atoi(v);
                             } else if (!strcmp(gtpu_key, "dev")) {
                                 dev = ogs_yaml_iter_value(&gtpu_iter);
+                            } else if (!strcmp(gtpu_key, "option")) {
+                                rv = ogs_app_config_parse_sockopt(
+                                        &gtpu_iter, &option);
+                                if (rv != OGS_OK) return rv;
+                                is_option = true;
                             } else if (!strcmp(gtpu_key,
                                         "teid_range_indication")) {
                                 teid_range_indication =
@@ -348,9 +371,13 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
 
                         if (addr) {
                             if (ogs_app()->parameter.no_ipv4 == 0)
-                                ogs_socknode_add(&list, AF_INET, addr);
+                                ogs_socknode_add(
+                                    &list, AF_INET, addr,
+                                    is_option ? &option : NULL);
                             if (ogs_app()->parameter.no_ipv6 == 0)
-                                ogs_socknode_add(&list6, AF_INET6, addr);
+                                ogs_socknode_add(
+                                    &list6, AF_INET6, addr,
+                                    is_option ? &option : NULL);
                             ogs_freeaddrinfo(addr);
                         }
 
@@ -358,7 +385,8 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                             rv = ogs_socknode_probe(
                                 ogs_app()->parameter.no_ipv4 ? NULL : &list,
                                 ogs_app()->parameter.no_ipv6 ? NULL : &list6,
-                                dev, port);
+                                dev, port,
+                                is_option ? &option : NULL);
                             ogs_assert(rv == OGS_OK);
                         }
 
@@ -452,9 +480,9 @@ int ogs_gtp_context_parse_config(const char *local, const char *remote)
                         ogs_list_init(&list6);
 
                         rv = ogs_socknode_probe(
-                            ogs_app()->parameter.no_ipv4 ? NULL : &list,
-                            ogs_app()->parameter.no_ipv6 ? NULL : &list6,
-                            NULL, self.gtpu_port);
+                                ogs_app()->parameter.no_ipv4 ? NULL : &list,
+                                ogs_app()->parameter.no_ipv6 ? NULL : &list6,
+                                NULL, self.gtpu_port, NULL);
                         ogs_assert(rv == OGS_OK);
 
                         /*
@@ -524,7 +552,7 @@ void ogs_gtp_node_free(ogs_gtp_node_t *node)
 }
 
 ogs_gtp_node_t *ogs_gtp_node_add_by_f_teid(
-        ogs_list_t *list, ogs_gtp_f_teid_t *f_teid, uint16_t port)
+        ogs_list_t *list, ogs_gtp2_f_teid_t *f_teid, uint16_t port)
 {
     int rv;
     ogs_gtp_node_t *node = NULL;
@@ -534,7 +562,7 @@ ogs_gtp_node_t *ogs_gtp_node_add_by_f_teid(
     ogs_assert(f_teid);
     ogs_assert(port);
 
-    rv = ogs_gtp_f_teid_to_sockaddr(f_teid, port, &addr);
+    rv = ogs_gtp2_f_teid_to_sockaddr(f_teid, port, &addr);
     ogs_expect_or_return_val(rv == OGS_OK, NULL);
 
     rv = ogs_filter_ip_version(
@@ -544,13 +572,15 @@ ogs_gtp_node_t *ogs_gtp_node_add_by_f_teid(
             ogs_app()->parameter.prefer_ipv4);
     ogs_assert(addr);
 
+#if 0 /* deprecated */
     rv = ogs_socknode_fill_scope_id_in_local(addr);
     ogs_assert(rv == OGS_OK);
+#endif
 
     node = ogs_gtp_node_new(addr);
     ogs_assert(node);
 
-    rv = ogs_gtp_f_teid_to_ip(f_teid, &node->ip);
+    rv = ogs_gtp2_f_teid_to_ip(f_teid, &node->ip);
     ogs_expect_or_return_val(rv == OGS_OK, NULL);
 
     ogs_list_add(list, node);
@@ -589,7 +619,7 @@ void ogs_gtp_node_remove(ogs_list_t *list, ogs_gtp_node_t *node)
 void ogs_gtp_node_remove_all(ogs_list_t *list)
 {
     ogs_gtp_node_t *node = NULL, *next_node = NULL;
-    
+
     ogs_list_for_each_safe(list, next_node, node)
         ogs_gtp_node_remove(list, node);
 }
@@ -611,7 +641,7 @@ ogs_gtp_node_t *ogs_gtp_node_find_by_addr(
 }
 
 ogs_gtp_node_t *ogs_gtp_node_find_by_f_teid(
-        ogs_list_t *list, ogs_gtp_f_teid_t *f_teid)
+        ogs_list_t *list, ogs_gtp2_f_teid_t *f_teid)
 {
     int rv;
     ogs_gtp_node_t *node = NULL;
@@ -620,7 +650,7 @@ ogs_gtp_node_t *ogs_gtp_node_find_by_f_teid(
     ogs_assert(list);
     ogs_assert(f_teid);
 
-    rv = ogs_gtp_f_teid_to_ip(f_teid, &ip);
+    rv = ogs_gtp2_f_teid_to_ip(f_teid, &ip);
     ogs_assert(rv == OGS_OK);
 
     ogs_list_for_each(list, node) {
@@ -652,8 +682,10 @@ ogs_gtp_node_t *ogs_gtp_node_add_by_ip(
             ogs_app()->parameter.prefer_ipv4);
     ogs_expect_or_return_val(addr, NULL);
 
+#if 0 /* deprecated */
     rv = ogs_socknode_fill_scope_id_in_local(addr);
     ogs_expect_or_return_val(rv == OGS_OK, NULL);
+#endif
 
     node = ogs_gtp_node_new(addr);
     ogs_expect_or_return_val(node, NULL);
