@@ -288,7 +288,7 @@ ED2(uint8_t spare:5;,
 
 #define OGS_NAS_ATTACH_TYPE_EPS_ATTACH 1
 #define OGS_NAS_ATTACH_TYPE_COMBINED_EPS_IMSI_ATTACH 2
-#define OGS_NAS_ATTACH_TYPE_EPS_ERMERGENCY_ATTACH 3
+#define OGS_NAS_ATTACH_TYPE_EPS_EMERGENCY_ATTACH 3
 #define OGS_NAS_ATTACH_TYPE_RESERVED 4
 typedef struct ogs_nas_eps_attach_type_s {
 ED4(uint8_t tsc:1;,
