@@ -87,8 +87,8 @@ typedef struct ogs_pfcp_node_s {
     ogs_sock_t      *sock;          /* Socket Instance */
     ogs_sockaddr_t  addr;           /* Remote Address */
 
-    ogs_list_t      local_list;    
-    ogs_list_t      remote_list;   
+    ogs_list_t      local_list;
+    ogs_list_t      remote_list;
 
     ogs_fsm_t       sm;             /* A state machine */
     ogs_timer_t     *t_association; /* timer to retry to associate peer node */
@@ -96,7 +96,7 @@ typedef struct ogs_pfcp_node_s {
 
     uint16_t        tac[OGS_MAX_NUM_OF_TAI];
     uint8_t         num_of_tac;
-    const char*     dnn[OGS_MAX_DNN_LEN+1];
+    const char*     dnn[OGS_MAX_NUM_OF_DNN];
     uint8_t         num_of_dnn;
     uint32_t        e_cell_id[OGS_MAX_NUM_OF_CELL_ID];
     uint8_t         num_of_e_cell_id;
@@ -137,6 +137,9 @@ typedef struct ogs_pfcp_pdr_s {
     ogs_pfcp_object_t       obj;
     uint32_t                index;
 
+    ogs_lnode_t             to_create_node;
+    ogs_lnode_t             to_modify_node;
+
     struct {
         struct {
             int len;
@@ -169,7 +172,10 @@ typedef struct ogs_pfcp_pdr_s {
     uint8_t                 qfi;
 
     ogs_pfcp_far_t          *far;
-    ogs_pfcp_urr_t          *urr;
+
+    int                     num_of_urr;
+    ogs_pfcp_urr_t          *urr[OGS_MAX_NUM_OF_URR];
+
     ogs_pfcp_qer_t          *qer;
 
     int                     num_of_flow;
@@ -231,7 +237,7 @@ typedef struct ogs_pfcp_urr_s {
 
     ogs_pfcp_measurement_method_t meas_method;
     ogs_pfcp_reporting_triggers_t rep_triggers;
-
+    ogs_pfcp_measurement_information_t meas_info;
     ogs_pfcp_measurement_period_t meas_period;
 
     ogs_pfcp_volume_threshold_t vol_threshold;
