@@ -148,7 +148,7 @@ ED8(uint8_t treu:1;,
  *   with multiple values.
  * 6/6 FRRT Sxb, N4
  *   The UP function supports Framed Routing
- *   (see IETF RFC 2865 [37] and IETF RFC 3162 [38]). 
+ *   (see IETF RFC 2865 [37] and IETF RFC 3162 [38]).
  * 6/5 TRACE Sxa, Sxb, Sxc, N4
  *   The UP function supports Trace (see clause 5.15).
  * 6/4 QUOAC Sxb, Sxc, N4
@@ -216,10 +216,10 @@ ED8(uint8_t gcom:1;,
  * 8/7 TSCU N4
  *   Time Sensitive Communication is supported by the UPF (see clause 5.26).
  * 8/6 IP6PL N4
- *   UPF supports: 
+ *   UPF supports:
  *     - UE IPv6 address(es) allocation with IPv6 prefix length other than
  *       default /64 (including allocating /128 individual IPv6 addresses),
- *       as specified in clause 4.6.2.2 of of 3GPP TS 23.316 [57]; and 
+ *       as specified in clause 4.6.2.2 of of 3GPP TS 23.316 [57]; and
  *     - multiple UE IPv6 addresses allocation using multiple instances
  *       of the UE IP Address IE in a same PDI or Traffic Endpoint,
  *       or using multiple PDIs or Traffic Endpoints
@@ -294,7 +294,7 @@ ED2(uint8_t reserved:7;,
     };
 } __attribute__ ((packed)) ogs_pfcp_up_function_features_t;
 
-/* 
+/*
  * 8.2.26 Apply Action
  *
  * Bit 1 – DROP (Drop): when set to 1, this indicates a request
@@ -845,6 +845,44 @@ ED3(uint8_t     spare:4;,
     };
 } __attribute__ ((packed)) ogs_pfcp_gate_status_t;
 
+/* 8.2.19 Reporting Triggers
+ */
+typedef struct ogs_pfcp_reporting_triggers_s {
+    union {
+        struct {
+ED8(uint8_t linked_usage_reporting:1;,
+    uint8_t dropped_dl_traffic_threshold:1;,
+    uint8_t stop_of_traffic:1;,
+    uint8_t start_of_traffic:1;,
+    uint8_t quota_holding_time:1;,
+    uint8_t time_threshold:1;,
+    uint8_t volume_threshold:1;,
+    uint8_t periodic_reporting:1;)
+        };
+        uint8_t reptri_5;
+    };
+    union {
+        struct {
+ED8(uint8_t quota_validity_time:1;,
+    uint8_t ip_multicast_join_leave:1;,
+    uint8_t event_quota:1;,
+    uint8_t event_threshold:1;,
+    uint8_t mac_addresses_reporting:1;,
+    uint8_t envelope_closure:1;,
+    uint8_t time_quota:1;,
+    uint8_t volume_quota:1;)
+        };
+        uint8_t reptri_6;
+    };
+    union {
+        struct {
+ED2(uint8_t spare:7;,
+    uint8_t report_the_end_marker_reception:1;)
+        };
+        uint8_t reptri_7;
+    };
+} __attribute__ ((packed)) ogs_pfcp_reporting_triggers_t;
+
 /*
  * 8.2.21 Report Type
  *
@@ -945,10 +983,10 @@ typedef uint8_t ogs_pfcp_measurement_method_t;
  * The Usage Report Trigger IE shall be encoded as shown in Figure 8.2.41-1.
  * It indicates the trigger of the usage report.
  */
-typedef struct ogs_pfcp_reporting_triggers_s {
+typedef struct ogs_pfcp_usage_report_trigger_s {
     union {
         struct {
-ED8(uint8_t linked_usage_reporting:1;,
+ED8(uint8_t immediate_report:1;;,
     uint8_t dropped_dl_traffic_threshold:1;,
     uint8_t stop_of_traffic:1;,
     uint8_t start_of_traffic:1;,
@@ -961,12 +999,12 @@ ED8(uint8_t linked_usage_reporting:1;,
     };
     union {
         struct {
-ED8(uint8_t quota_validity_time:1;,
-    uint8_t ip_multicast_join_leave:1;,
-    uint8_t event_quota:1;,
-    uint8_t event_threshold:1;,
+ED8(uint8_t event_threshold:1;,
     uint8_t mac_addresses_reporting:1;,
     uint8_t envelope_closure:1;,
+    uint8_t monitoring_time:1;,
+    uint8_t termination_report:1;,
+    uint8_t linked_usage_reporting:1;,
     uint8_t time_quota:1;,
     uint8_t volume_quota:1;)
         };
@@ -974,12 +1012,16 @@ ED8(uint8_t quota_validity_time:1;,
     };
     union {
         struct {
-ED2(uint8_t spare:7;,
-    uint8_t report_the_end_marker_reception:1;)
+ED6(uint8_t spare:3;,
+    uint8_t report_the_end_marker_reception:1;,
+    uint8_t quota_validity_time:1;,
+    uint8_t ip_multicast_join_leave:1;,
+    uint8_t termination_by_up_function_report:1;,
+    uint8_t event_quota:1;)
         };
         uint8_t reptri_7;
     };
-} __attribute__ ((packed)) ogs_pfcp_reporting_triggers_t;
+} __attribute__ ((packed)) ogs_pfcp_usage_report_trigger_t;
 
 /*
  * 8.2.42 Measurement Period
@@ -1125,91 +1167,91 @@ int16_t ogs_pfcp_parse_dropped_dl_traffic_threshold(
         ogs_pfcp_dropped_dl_traffic_threshold_t *threshold,
         ogs_tlv_octet_t *octet);
 
-/** 8.2.71 UR-SEQN 
- * 
+/** 8.2.71 UR-SEQN
+ *
  * The UR-SEQN (Usage Report Sequence Number) IE identifies the order
  * in which a usage report is generated for a
- * given URR. It shall be encoded as shown in Figure 8.2.71-1. 
- * 
+ * given URR. It shall be encoded as shown in Figure 8.2.71-1.
+ *
  * The UR-SEQN value shall be encoded as an Unsigned32 binary integer value.
 */
 typedef uint32_t ogs_pfcp_urr_ur_seqn_t;
 
 /** 8.2.52 Start Time
- * 
+ *
  * The Start Time IE indicates the time at which the UP function started
  * to collect the charging information. It shall be encoded as shown
  * in Figure 8.2.52-1.
- * 
+ *
  * The Start Time field shall contain a UTC time. Octets 5 to 8 shall
  * be encoded in the same format as the first four octets
  * of the 64-bit timestamp format as defined in clause 6 of IETF RFC 5905 [12].
- * 
+ *
  * NOTE: The encoding is defined as the time in seconds relative to 00:00:00
  * on 1 January 1900.
  */
 typedef uint32_t ogs_pfcp_start_time_t;
 
-/** 8.2.53 End Time 
+/** 8.2.53 End Time
  * The End Time IE indicates the time at which the UP function ended
  * to collect the charging information. It shall be encoded as shown
  * in Figure 8.2.53-1.
- * 
+ *
  * The End Time field shall contain a UTC time. Octets 5 to 8 shall be
  * encoded in the same format as the first four octets of the 64-bit timestamp
  * format as defined in clause 6 of IETF RFC 5905 [12].
- * 
+ *
  * NOTE: The encoding is defined as the time in seconds relative to 00:00:00
  * on 1 January 1900.
 */
 typedef uint32_t ogs_pfcp_end_time_t;
 
-/** 8.2.45 Duration Measurement 
- * 
+/** 8.2.45 Duration Measurement
+ *
  * The Duration Measurement IE type shall be encoded as shown
  * in Figure 8.2.45-1. It contains the used time in seconds
- * 
- * The Duration value shall be encoded as an Unsigned32 binary integer value. 
+ *
+ * The Duration value shall be encoded as an Unsigned32 binary integer value.
 */
 typedef uint32_t ogs_pfcp_duration_measurement_t;
 
-/** 8.2.46 Time of First Packet 
- * 
+/** 8.2.46 Time of First Packet
+ *
  * The Time of First Packet IE indicates the time stamp for the first
  * IP packet transmitted for a given usage report. It shall be encoded
  * as shown in Figure 8.2.46-1.
- * 
+ *
  * The End Time field shall contain a UTC time. Octets 5 to 8 shall
  * be encoded in the same format as the first four octets
- * of the 64-bit timestamp format as defined in clause 6 of IETF RFC 5905 [12]. 
- * 
+ * of the 64-bit timestamp format as defined in clause 6 of IETF RFC 5905 [12].
+ *
  * NOTE: The encoding is defined as the time in seconds relative to 00:00:00
  * on 1 January 1900.
 */
 typedef uint32_t ogs_pfcp_time_of_first_packet_t;
 
-/** 8.2.47 Time of Last Packet 
- * 
+/** 8.2.47 Time of Last Packet
+ *
  * The Time of Last Packet IE indicates the time stamp for the last
  * IP packet transmitted for a given usage report. It shall be encoded
  * as shown in Figure 8.2.47-1.
- * 
+ *
  * The End Time field shall contain a UTC time. Octets 5 to 8 shall
  * be encoded in the same format as the first four octets
- * of the 64-bit timestamp format as defined in clause 6 of IETF RFC 5905 [12]. 
- * 
+ * of the 64-bit timestamp format as defined in clause 6 of IETF RFC 5905 [12].
+ *
  * NOTE: The encoding is defined as the time in seconds relative to 00:00:00
  * on 1 January 1900.
 */
 typedef uint32_t ogs_pfcp_time_of_last_packet_t;
 
-/** 8.2.44 Volume Measurement 
- * 
+/** 8.2.44 Volume Measurement
+ *
  * The Volume Measurement IE contains the measured traffic volumes.
  * It shall be encoded as shown in Figure 8.2.44-1.
- * 
+ *
  * The following flags are coded within Octet 5:
- * 
+ *
  * - Bit 1 – TOVOL: If this bit is set to "1", then the Total Volume field
  *   shall be present, otherwise the Total Volume field shall not be present.
  * - Bit 2 – ULVOL: If this bit is set to "1", then the Uplink Volume field
@@ -1226,9 +1268,9 @@ typedef uint32_t ogs_pfcp_time_of_last_packet_t;
  *   of Packets field shall be present, otherwise the Downlink Number
  *   of Packets field shall not be present.
  * - Bit 7 to bit 8: Spare, for future use and set to "0".
- * 
+ *
  * At least one bit shall be set to "1". Several bits may be set to "1".
- * 
+ *
  * The Total Volume, Uplink Volume and Downlink Volume fields shall be encoded
  * as an Unsigned64 binary integer value. They shall contain the total,
  * uplink or downlink number of octets respectively
@@ -1255,6 +1297,23 @@ ED7(uint8_t spare:2;,
     uint64_t downlink_n_packets;
 } __attribute__ ((packed)) ogs_pfcp_volume_measurement_t;
 
+/*
+ * 8.2.68 Measurement Information
+ */
+typedef struct ogs_pfcp_measurement_information_s {
+    union {
+        struct {
+ED6(uint8_t spare:3;,
+    uint8_t mnop:1;,
+    uint8_t istm:1;,
+    uint8_t radi:1;,
+    uint8_t inam:1;,
+    uint8_t mbqe:1;)
+        };
+    uint8_t octet5;
+    };
+} __attribute__ ((packed)) ogs_pfcp_measurement_information_t;
+
 typedef struct ogs_pfcp_user_plane_report_s {
     ogs_pfcp_report_type_t type;
     struct {
@@ -1265,14 +1324,15 @@ typedef struct ogs_pfcp_user_plane_report_s {
     struct {
         ogs_pfcp_urr_id_t id;
         ogs_pfcp_urr_ur_seqn_t seqn;
-        ogs_pfcp_reporting_triggers_t rep_triggers;
+        ogs_pfcp_usage_report_trigger_t rep_trigger;
         ogs_pfcp_start_time_t start_time;
         ogs_pfcp_end_time_t end_time;
         ogs_pfcp_volume_measurement_t vol_measurement;
         ogs_pfcp_duration_measurement_t dur_measurement;
         ogs_pfcp_time_of_first_packet_t time_of_first_packet;
         ogs_pfcp_time_of_last_packet_t time_of_last_packet;
-    } usage_report;
+    } usage_report [OGS_MAX_NUM_OF_URR];
+    unsigned int num_of_usage_report;
     struct {
         ogs_pfcp_f_teid_t remote_f_teid;
         int remote_f_teid_len;
