@@ -33,11 +33,13 @@ extern "C" {
  */
 typedef struct ogs_pfcp_xact_s {
     ogs_lnode_t     lnode;          /**< A node of list */
+    ogs_lnode_t     tmpnode;        /**< A node of temp-list */
+
     ogs_index_t     index;
-    
+
 #define OGS_PFCP_LOCAL_ORIGINATOR  0
 #define OGS_PFCP_REMOTE_ORIGINATOR 1
-    uint8_t         org;            /**< Transaction' originator. 
+    uint8_t         org;            /**< Transaction' originator.
                                          local or remote */
 
     uint32_t        xid;            /**< Transaction ID */
@@ -48,8 +50,8 @@ typedef struct ogs_pfcp_xact_s {
     void            *data;
 
     int             step;           /**< Current step in the sequence.
-                                         1 : Initial 
-                                         2 : Triggered 
+                                         1 : Initial
+                                         2 : Triggered
                                          3 : Triggered-Reply */
     struct {
         uint8_t     type;           /**< Message type history */
@@ -97,23 +99,31 @@ typedef struct ogs_pfcp_xact_s {
 #define OGS_PFCP_MODIFY_XN_HANDOVER ((uint64_t)1<<21)
 #define OGS_PFCP_MODIFY_N2_HANDOVER ((uint64_t)1<<22)
 #define OGS_PFCP_MODIFY_HANDOVER_CANCEL ((uint64_t)1<<23)
+#define OGS_PFCP_MODIFY_URR  ((uint64_t)1<<24) /* type of trigger */
+#define OGS_PFCP_MODIFY_URR_MEAS_METHOD ((uint64_t)1<<25)
+#define OGS_PFCP_MODIFY_URR_REPORT_TRIGGER ((uint64_t)1<<26)
+#define OGS_PFCP_MODIFY_URR_VOLUME_THRESH ((uint64_t)1<<27)
+#define OGS_PFCP_MODIFY_URR_TIME_THRESH ((uint64_t)1<<28)
 
     uint64_t        modify_flags;
 
-#define OGS_PFCP_DELETE_TRIGGER_UE_REQUESTED 1
-#define OGS_PFCP_DELETE_TRIGGER_PCF_INITIATED 2
-#define OGS_PFCP_DELETE_TRIGGER_RAN_INITIATED 3
-#define OGS_PFCP_DELETE_TRIGGER_SMF_INITIATED 4
-#define OGS_PFCP_DELETE_TRIGGER_AMF_RELEASE_SM_CONTEXT 5
-#define OGS_PFCP_DELETE_TRIGGER_AMF_UPDATE_SM_CONTEXT 6
+#define OGS_PFCP_DELETE_TRIGGER_LOCAL_INITIATED 1
+#define OGS_PFCP_DELETE_TRIGGER_UE_REQUESTED 2
+#define OGS_PFCP_DELETE_TRIGGER_PCF_INITIATED 3
+#define OGS_PFCP_DELETE_TRIGGER_RAN_INITIATED 4
+#define OGS_PFCP_DELETE_TRIGGER_SMF_INITIATED 5
+#define OGS_PFCP_DELETE_TRIGGER_AMF_RELEASE_SM_CONTEXT 6
+#define OGS_PFCP_DELETE_TRIGGER_AMF_UPDATE_SM_CONTEXT 7
     int             delete_trigger;
+
+    ogs_list_t      pdr_to_create_list;
+    ogs_list_t      bearer_to_modify_list;
 } ogs_pfcp_xact_t;
 
 int ogs_pfcp_xact_init(void);
 void ogs_pfcp_xact_final(void);
 
 ogs_pfcp_xact_t *ogs_pfcp_xact_local_create(ogs_pfcp_node_t *node,
-        ogs_pfcp_header_t *hdesc, ogs_pkbuf_t *pkbuf,
         void (*cb)(ogs_pfcp_xact_t *xact, void *data), void *data);
 ogs_pfcp_xact_t *ogs_pfcp_xact_cycle(ogs_pfcp_xact_t *xact);
 void ogs_pfcp_xact_delete_all(ogs_pfcp_node_t *node);
