@@ -268,13 +268,11 @@ int ogs_strftimezone(char *str, size_t size, int tm_gmtoff)
     }
 
     len = ogs_snprintf(str, size, "%c%02d:%02d",
-            off_sign, off / 3600, off % 3600);
-
+            off_sign, off / 3600, (off % 3600) / 60);
     if (len != 6) {
-        ogs_warn("Unknown tm_gmtoff[%d:%d]", tm_gmtoff, off);
-
-        len = ogs_snprintf(str, size, "%c%02d:%02d",
-                off_sign, (off / 3600) % 100, (off % 3600) % 100);
+        ogs_fatal("Unknown tm_gmtoff[%d:%d], len[%d], str[%s]",
+                tm_gmtoff, off, len, str);
+        ogs_assert_if_reached();
     }
 
     return len;
