@@ -9,10 +9,9 @@ OpenAPI_am_policy_data_t *OpenAPI_am_policy_data_create(
     OpenAPI_list_t *subsc_cats
 )
 {
-    OpenAPI_am_policy_data_t *am_policy_data_local_var = OpenAPI_malloc(sizeof(OpenAPI_am_policy_data_t));
-    if (!am_policy_data_local_var) {
-        return NULL;
-    }
+    OpenAPI_am_policy_data_t *am_policy_data_local_var = ogs_malloc(sizeof(OpenAPI_am_policy_data_t));
+    ogs_assert(am_policy_data_local_var);
+
     am_policy_data_local_var->pra_infos = pra_infos;
     am_policy_data_local_var->subsc_cats = subsc_cats;
 
@@ -27,6 +26,7 @@ void OpenAPI_am_policy_data_free(OpenAPI_am_policy_data_t *am_policy_data)
     OpenAPI_lnode_t *node;
     OpenAPI_list_for_each(am_policy_data->pra_infos, node) {
         OpenAPI_map_t *localKeyValue = (OpenAPI_map_t*)node->data;
+        ogs_free(localKeyValue->key);
         OpenAPI_presence_info_free(localKeyValue->value);
         ogs_free(localKeyValue);
     }
@@ -109,9 +109,9 @@ OpenAPI_am_policy_data_t *OpenAPI_am_policy_data_parseFromJSON(cJSON *am_policy_
         cJSON *localMapObject = pra_infos_local_map;
         if (cJSON_IsObject(pra_infos_local_map)) {
             localMapKeyPair = OpenAPI_map_create(
-                localMapObject->string, OpenAPI_presence_info_parseFromJSON(localMapObject));
+                ogs_strdup(localMapObject->string), OpenAPI_presence_info_parseFromJSON(localMapObject));
         } else if (cJSON_IsNull(pra_infos_local_map)) {
-            localMapKeyPair = OpenAPI_map_create(localMapObject->string, NULL);
+            localMapKeyPair = OpenAPI_map_create(ogs_strdup(localMapObject->string), NULL);
         } else {
             ogs_error("OpenAPI_am_policy_data_parseFromJSON() failed [pra_infos]");
             goto end;
@@ -136,7 +136,7 @@ OpenAPI_am_policy_data_t *OpenAPI_am_policy_data_parseFromJSON(cJSON *am_policy_
         ogs_error("OpenAPI_am_policy_data_parseFromJSON() failed [subsc_cats]");
         goto end;
     }
-    OpenAPI_list_add(subsc_catsList , ogs_strdup_or_assert(subsc_cats_local->valuestring));
+    OpenAPI_list_add(subsc_catsList , ogs_strdup(subsc_cats_local->valuestring));
     }
     }
 
