@@ -1,7 +1,7 @@
 /*
  * amf_event_trigger.h
  *
- *
+ * 
  */
 
 #ifndef _OpenAPI_amf_event_trigger_H_
@@ -12,6 +12,7 @@
 #include "../include/list.h"
 #include "../include/keyValuePair.h"
 #include "../include/binary.h"
+#include "amf_event_trigger_any_of.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -22,7 +23,7 @@ typedef struct OpenAPI_amf_event_trigger_s {
 } OpenAPI_amf_event_trigger_t;
 
 OpenAPI_amf_event_trigger_t *OpenAPI_amf_event_trigger_create(
-    );
+);
 void OpenAPI_amf_event_trigger_free(OpenAPI_amf_event_trigger_t *amf_event_trigger);
 OpenAPI_amf_event_trigger_t *OpenAPI_amf_event_trigger_parseFromJSON(cJSON *amf_event_triggerJSON);
 cJSON *OpenAPI_amf_event_trigger_convertToJSON(OpenAPI_amf_event_trigger_t *amf_event_trigger);
