@@ -27,6 +27,7 @@ typedef struct OpenAPI_dnn_upf_info_item_s {
     OpenAPI_list_t *pdu_session_types;
     OpenAPI_list_t *ipv4_address_ranges;
     OpenAPI_list_t *ipv6_prefix_ranges;
+    OpenAPI_list_t* dnai_nw_instance_list;
 } OpenAPI_dnn_upf_info_item_t;
 
 OpenAPI_dnn_upf_info_item_t *OpenAPI_dnn_upf_info_item_create(
@@ -34,8 +35,9 @@ OpenAPI_dnn_upf_info_item_t *OpenAPI_dnn_upf_info_item_create(
     OpenAPI_list_t *dnai_list,
     OpenAPI_list_t *pdu_session_types,
     OpenAPI_list_t *ipv4_address_ranges,
-    OpenAPI_list_t *ipv6_prefix_ranges
-    );
+    OpenAPI_list_t *ipv6_prefix_ranges,
+    OpenAPI_list_t* dnai_nw_instance_list
+);
 void OpenAPI_dnn_upf_info_item_free(OpenAPI_dnn_upf_info_item_t *dnn_upf_info_item);
 OpenAPI_dnn_upf_info_item_t *OpenAPI_dnn_upf_info_item_parseFromJSON(cJSON *dnn_upf_info_itemJSON);
 cJSON *OpenAPI_dnn_upf_info_item_convertToJSON(OpenAPI_dnn_upf_info_item_t *dnn_upf_info_item);
