@@ -1,7 +1,7 @@
 /*
  * exemption_ind.h
  *
- *
+ * 
  */
 
 #ifndef _OpenAPI_exemption_ind_H_
@@ -19,16 +19,22 @@ extern "C" {
 
 typedef struct OpenAPI_exemption_ind_s OpenAPI_exemption_ind_t;
 typedef struct OpenAPI_exemption_ind_s {
+    bool is_dnn_congestion;
     int dnn_congestion;
+    bool is_snssai_only_congestion;
     int snssai_only_congestion;
+    bool is_snssai_dnn_congestion;
     int snssai_dnn_congestion;
 } OpenAPI_exemption_ind_t;
 
 OpenAPI_exemption_ind_t *OpenAPI_exemption_ind_create(
+    bool is_dnn_congestion,
     int dnn_congestion,
+    bool is_snssai_only_congestion,
     int snssai_only_congestion,
+    bool is_snssai_dnn_congestion,
     int snssai_dnn_congestion
-    );
+);
 void OpenAPI_exemption_ind_free(OpenAPI_exemption_ind_t *exemption_ind);
 OpenAPI_exemption_ind_t *OpenAPI_exemption_ind_parseFromJSON(cJSON *exemption_indJSON);
 cJSON *OpenAPI_exemption_ind_convertToJSON(OpenAPI_exemption_ind_t *exemption_ind);
