@@ -263,6 +263,12 @@ OpenAPI_ext_problem_details_t *OpenAPI_ext_problem_details_parseFromJSON(cJSON *
         }
         OpenAPI_invalid_param_t *invalid_paramsItem = OpenAPI_invalid_param_parseFromJSON(invalid_params_local_nonprimitive);
 
+        if (!invalid_paramsItem) {
+            ogs_error("No invalid_paramsItem");
+            OpenAPI_list_free(invalid_paramsList);
+            goto end;
+        }
+
         OpenAPI_list_add(invalid_paramsList, invalid_paramsItem);
     }
     }
