@@ -15,10 +15,9 @@ OpenAPI_hsmf_update_error_t *OpenAPI_hsmf_update_error_create(
     char *recovery_time
 )
 {
-    OpenAPI_hsmf_update_error_t *hsmf_update_error_local_var = OpenAPI_malloc(sizeof(OpenAPI_hsmf_update_error_t));
-    if (!hsmf_update_error_local_var) {
-        return NULL;
-    }
+    OpenAPI_hsmf_update_error_t *hsmf_update_error_local_var = ogs_malloc(sizeof(OpenAPI_hsmf_update_error_t));
+    ogs_assert(hsmf_update_error_local_var);
+
     hsmf_update_error_local_var->error = error;
     hsmf_update_error_local_var->is_pti = is_pti;
     hsmf_update_error_local_var->pti = pti;
@@ -169,11 +168,11 @@ OpenAPI_hsmf_update_error_t *OpenAPI_hsmf_update_error_parseFromJSON(cJSON *hsmf
         error_local_nonprim,
         pti ? true : false,
         pti ? pti->valuedouble : 0,
-        n1sm_cause ? ogs_strdup_or_assert(n1sm_cause->valuestring) : NULL,
+        n1sm_cause ? ogs_strdup(n1sm_cause->valuestring) : NULL,
         n1_sm_info_to_ue ? n1_sm_info_to_ue_local_nonprim : NULL,
         back_off_timer ? true : false,
         back_off_timer ? back_off_timer->valuedouble : 0,
-        recovery_time ? ogs_strdup_or_assert(recovery_time->valuestring) : NULL
+        recovery_time ? ogs_strdup(recovery_time->valuestring) : NULL
     );
 
     return hsmf_update_error_local_var;
