@@ -222,6 +222,12 @@ OpenAPI_nwdaf_cond_t *OpenAPI_nwdaf_cond_parseFromJSON(cJSON *nwdaf_condJSON)
         }
         OpenAPI_snssai_t *snssai_listItem = OpenAPI_snssai_parseFromJSON(snssai_list_local_nonprimitive);
 
+        if (!snssai_listItem) {
+            ogs_error("No snssai_listItem");
+            OpenAPI_list_free(snssai_listList);
+            goto end;
+        }
+
         OpenAPI_list_add(snssai_listList, snssai_listItem);
     }
     }
@@ -245,6 +251,12 @@ OpenAPI_nwdaf_cond_t *OpenAPI_nwdaf_cond_parseFromJSON(cJSON *nwdaf_condJSON)
         }
         OpenAPI_tai_t *tai_listItem = OpenAPI_tai_parseFromJSON(tai_list_local_nonprimitive);
 
+        if (!tai_listItem) {
+            ogs_error("No tai_listItem");
+            OpenAPI_list_free(tai_listList);
+            goto end;
+        }
+
         OpenAPI_list_add(tai_listList, tai_listItem);
     }
     }
@@ -268,6 +280,12 @@ OpenAPI_nwdaf_cond_t *OpenAPI_nwdaf_cond_parseFromJSON(cJSON *nwdaf_condJSON)
         }
         OpenAPI_tai_range_t *tai_range_listItem = OpenAPI_tai_range_parseFromJSON(tai_range_list_local_nonprimitive);
 
+        if (!tai_range_listItem) {
+            ogs_error("No tai_range_listItem");
+            OpenAPI_list_free(tai_range_listList);
+            goto end;
+        }
+
         OpenAPI_list_add(tai_range_listList, tai_range_listItem);
     }
     }
