@@ -1,7 +1,7 @@
 /*
  * pdu_session_created_data.h
  *
- *
+ * 
  */
 
 #ifndef _OpenAPI_pdu_session_created_data_H_
@@ -19,6 +19,7 @@
 #include "max_integrity_protected_data_rate.h"
 #include "pdu_session_type.h"
 #include "qos_flow_setup_item.h"
+#include "redundant_pdu_session_information.h"
 #include "ref_to_binary_data.h"
 #include "roaming_charging_profile.h"
 #include "snssai.h"
@@ -40,8 +41,10 @@ typedef struct OpenAPI_pdu_session_created_data_s {
     OpenAPI_list_t *qos_flows_setup_list;
     char *h_smf_instance_id;
     char *smf_instance_id;
+    bool is_pdu_session_id;
     int pdu_session_id;
     struct OpenAPI_snssai_s *s_nssai;
+    bool is_enable_pause_charging;
     int enable_pause_charging;
     char *ue_ipv4_address;
     char *ue_ipv6_prefix;
@@ -51,6 +54,7 @@ typedef struct OpenAPI_pdu_session_created_data_s {
     char *supported_features;
     OpenAPI_max_integrity_protected_data_rate_e max_integrity_protected_data_rate;
     OpenAPI_max_integrity_protected_data_rate_e max_integrity_protected_data_rate_dl;
+    bool is_always_on_granted;
     int always_on_granted;
     char *gpsi;
     struct OpenAPI_up_security_s *up_security;
@@ -59,14 +63,20 @@ typedef struct OpenAPI_pdu_session_created_data_s {
     char *smf_service_instance_id;
     char *recovery_time;
     OpenAPI_list_t *dnai_list;
+    bool is_ipv6_multi_homing_ind;
     int ipv6_multi_homing_ind;
+    bool is_ma_accepted_ind;
     int ma_accepted_ind;
     char *home_provided_charging_id;
+    bool is_nef_ext_buf_support_ind;
     int nef_ext_buf_support_ind;
+    bool is_small_data_rate_control_enabled;
     int small_data_rate_control_enabled;
     char *ue_ipv6_interface_id;
+    bool is_ipv6_index;
     int ipv6_index;
     struct OpenAPI_ip_address_s *dn_aaa_address;
+    struct OpenAPI_redundant_pdu_session_information_s *redundant_pdu_session_info;
 } OpenAPI_pdu_session_created_data_t;
 
 OpenAPI_pdu_session_created_data_t *OpenAPI_pdu_session_created_data_create(
@@ -79,8 +89,10 @@ OpenAPI_pdu_session_created_data_t *OpenAPI_pdu_session_created_data_create(
     OpenAPI_list_t *qos_flows_setup_list,
     char *h_smf_instance_id,
     char *smf_instance_id,
+    bool is_pdu_session_id,
     int pdu_session_id,
     OpenAPI_snssai_t *s_nssai,
+    bool is_enable_pause_charging,
     int enable_pause_charging,
     char *ue_ipv4_address,
     char *ue_ipv6_prefix,
@@ -90,6 +102,7 @@ OpenAPI_pdu_session_created_data_t *OpenAPI_pdu_session_created_data_create(
     char *supported_features,
     OpenAPI_max_integrity_protected_data_rate_e max_integrity_protected_data_rate,
     OpenAPI_max_integrity_protected_data_rate_e max_integrity_protected_data_rate_dl,
+    bool is_always_on_granted,
     int always_on_granted,
     char *gpsi,
     OpenAPI_up_security_t *up_security,
@@ -98,15 +111,21 @@ OpenAPI_pdu_session_created_data_t *OpenAPI_pdu_session_created_data_create(
     char *smf_service_instance_id,
     char *recovery_time,
     OpenAPI_list_t *dnai_list,
+    bool is_ipv6_multi_homing_ind,
     int ipv6_multi_homing_ind,
+    bool is_ma_accepted_ind,
     int ma_accepted_ind,
     char *home_provided_charging_id,
+    bool is_nef_ext_buf_support_ind,
     int nef_ext_buf_support_ind,
+    bool is_small_data_rate_control_enabled,
     int small_data_rate_control_enabled,
     char *ue_ipv6_interface_id,
+    bool is_ipv6_index,
     int ipv6_index,
-    OpenAPI_ip_address_t *dn_aaa_address
-    );
+    OpenAPI_ip_address_t *dn_aaa_address,
+    OpenAPI_redundant_pdu_session_information_t *redundant_pdu_session_info
+);
 void OpenAPI_pdu_session_created_data_free(OpenAPI_pdu_session_created_data_t *pdu_session_created_data);
 OpenAPI_pdu_session_created_data_t *OpenAPI_pdu_session_created_data_parseFromJSON(cJSON *pdu_session_created_dataJSON);
 cJSON *OpenAPI_pdu_session_created_data_convertToJSON(OpenAPI_pdu_session_created_data_t *pdu_session_created_data);
