@@ -1,7 +1,7 @@
 /*
  * port_management_container.h
  *
- *
+ * 
  */
 
 #ifndef _OpenAPI_port_management_container_H_
@@ -12,7 +12,6 @@
 #include "../include/list.h"
 #include "../include/keyValuePair.h"
 #include "../include/binary.h"
-#include "tsn_port_identifier.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -21,13 +20,13 @@ extern "C" {
 typedef struct OpenAPI_port_management_container_s OpenAPI_port_management_container_t;
 typedef struct OpenAPI_port_management_container_s {
     char port_man_cont;
-    struct OpenAPI_tsn_port_identifier_s *port_identifier;
+    int port_num;
 } OpenAPI_port_management_container_t;
 
 OpenAPI_port_management_container_t *OpenAPI_port_management_container_create(
     char port_man_cont,
-    OpenAPI_tsn_port_identifier_t *port_identifier
-    );
+    int port_num
+);
 void OpenAPI_port_management_container_free(OpenAPI_port_management_container_t *port_management_container);
 OpenAPI_port_management_container_t *OpenAPI_port_management_container_parseFromJSON(cJSON *port_management_containerJSON);
 cJSON *OpenAPI_port_management_container_convertToJSON(OpenAPI_port_management_container_t *port_management_container);
