@@ -9,10 +9,9 @@ OpenAPI_ptw_parameters_t *OpenAPI_ptw_parameters_create(
     char *ptw_value
 )
 {
-    OpenAPI_ptw_parameters_t *ptw_parameters_local_var = OpenAPI_malloc(sizeof(OpenAPI_ptw_parameters_t));
-    if (!ptw_parameters_local_var) {
-        return NULL;
-    }
+    OpenAPI_ptw_parameters_t *ptw_parameters_local_var = ogs_malloc(sizeof(OpenAPI_ptw_parameters_t));
+    ogs_assert(ptw_parameters_local_var);
+
     ptw_parameters_local_var->operation_mode = operation_mode;
     ptw_parameters_local_var->ptw_value = ptw_value;
 
@@ -82,7 +81,7 @@ OpenAPI_ptw_parameters_t *OpenAPI_ptw_parameters_parseFromJSON(cJSON *ptw_parame
 
     ptw_parameters_local_var = OpenAPI_ptw_parameters_create (
         operation_modeVariable,
-        ogs_strdup_or_assert(ptw_value->valuestring)
+        ogs_strdup(ptw_value->valuestring)
     );
 
     return ptw_parameters_local_var;
