@@ -109,6 +109,12 @@ OpenAPI_snssai_smf_info_item_t *OpenAPI_snssai_smf_info_item_parseFromJSON(cJSON
         }
         OpenAPI_dnn_smf_info_item_t *dnn_smf_info_listItem = OpenAPI_dnn_smf_info_item_parseFromJSON(dnn_smf_info_list_local_nonprimitive);
 
+        if (!dnn_smf_info_listItem) {
+            ogs_error("No dnn_smf_info_listItem");
+            OpenAPI_list_free(dnn_smf_info_listList);
+            goto end;
+        }
+
         OpenAPI_list_add(dnn_smf_info_listList, dnn_smf_info_listItem);
     }
 
