@@ -1,7 +1,7 @@
 /*
  * up_path_chg_event.h
  *
- *
+ * 
  */
 
 #ifndef _OpenAPI_up_path_chg_event_H_
@@ -23,6 +23,7 @@ typedef struct OpenAPI_up_path_chg_event_s {
     char *notification_uri;
     char *notif_corre_id;
     OpenAPI_dnai_change_type_e dnai_chg_type;
+    bool is_af_ack_ind;
     int af_ack_ind;
 } OpenAPI_up_path_chg_event_t;
 
@@ -30,8 +31,9 @@ OpenAPI_up_path_chg_event_t *OpenAPI_up_path_chg_event_create(
     char *notification_uri,
     char *notif_corre_id,
     OpenAPI_dnai_change_type_e dnai_chg_type,
+    bool is_af_ack_ind,
     int af_ack_ind
-    );
+);
 void OpenAPI_up_path_chg_event_free(OpenAPI_up_path_chg_event_t *up_path_chg_event);
 OpenAPI_up_path_chg_event_t *OpenAPI_up_path_chg_event_parseFromJSON(cJSON *up_path_chg_eventJSON);
 cJSON *OpenAPI_up_path_chg_event_convertToJSON(OpenAPI_up_path_chg_event_t *up_path_chg_event);
