@@ -26,23 +26,27 @@
 extern "C" {
 #endif
 
-void ogs_sbi_send(ogs_sbi_nf_instance_t *nf_instance,
+bool ogs_sbi_send(ogs_sbi_nf_instance_t *nf_instance,
         ogs_sbi_client_cb_f client_cb, ogs_sbi_xact_t *xact);
 bool ogs_sbi_discover_and_send(ogs_sbi_xact_t *xact,
         ogs_fsm_handler_t nf_state_registered, ogs_sbi_client_cb_f client_cb);
 
-void ogs_nnrf_nfm_send_nf_update(ogs_sbi_nf_instance_t *nf_instance);
-void ogs_nnrf_nfm_send_nf_de_register(ogs_sbi_nf_instance_t *nf_instance);
+bool ogs_nnrf_nfm_send_nf_update(ogs_sbi_nf_instance_t *nf_instance);
+bool ogs_nnrf_nfm_send_nf_de_register(ogs_sbi_nf_instance_t *nf_instance);
 
-void ogs_nnrf_nfm_send_nf_status_subscribe(ogs_sbi_client_t *client,
+bool ogs_nnrf_nfm_send_nf_status_subscribe(ogs_sbi_client_t *client,
         OpenAPI_nf_type_e req_nf_type, char *req_nf_instance_id,
         OpenAPI_nf_type_e subscr_cond_nf_type);
-void ogs_nnrf_nfm_send_nf_status_unsubscribe(
+bool ogs_nnrf_nfm_send_nf_status_unsubscribe(
         ogs_sbi_subscription_t *subscription);
 
-void ogs_nnrf_disc_send_nf_discover(ogs_sbi_nf_instance_t *nf_instance,
+bool ogs_nnrf_disc_send_nf_discover(ogs_sbi_nf_instance_t *nf_instance,
         OpenAPI_nf_type_e target_nf_type, void *data);
 
+#define ogs_sbi_send_http_status_no_content(__sTREAM) \
+        ogs_sbi_send_response(__sTREAM, OGS_SBI_HTTP_STATUS_NO_CONTENT)
+bool ogs_sbi_send_response(ogs_sbi_stream_t *stream, int status);
+
 #ifdef __cplusplus
 }
 #endif
