@@ -113,18 +113,18 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
 
     ogs_nas_eps_message_t nas_message;
     enb_ue_t *enb_ue = NULL;
+    sgw_ue_t *sgw_ue = NULL;
     mme_ue_t *mme_ue = NULL;
 
     mme_bearer_t *bearer = NULL;
     mme_bearer_t *default_bearer = NULL;
     mme_sess_t *sess = NULL;
 
-    ogs_pkbuf_t *s6abuf = NULL;
     ogs_diam_s6a_message_t *s6a_message = NULL;
 
     ogs_gtp_node_t *gnode = NULL;
     ogs_gtp_xact_t *xact = NULL;
-    ogs_gtp_message_t gtp_message;
+    ogs_gtp2_message_t gtp_message;
 
     mme_vlr_t *vlr = NULL;
 
@@ -185,10 +185,13 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             ogs_free(addr);
         }
 
-        enb->max_num_of_ostreams =
-                ogs_min(max_num_of_ostreams, enb->max_num_of_ostreams);
+        if (enb->max_num_of_ostreams)
+            enb->max_num_of_ostreams =
+                    ogs_min(max_num_of_ostreams, enb->max_num_of_ostreams);
+        else
+            enb->max_num_of_ostreams = max_num_of_ostreams;
 
-        ogs_debug("eNB-S1 SCTP_COMM_UP[%s] Max Num of Outbound Streams[%d]", 
+        ogs_info("eNB-N2[%s] max_num_of_ostreams : %d",
             OGS_ADDR(enb->sctp.addr, buf), enb->max_num_of_ostreams);
 
         break;
@@ -241,7 +244,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             ogs_warn("Cannot decode S1AP message");
             ogs_assert(OGS_OK ==
                 s1ap_send_error_indication(
-                    enb, NULL, NULL, S1AP_Cause_PR_protocol, 
+                    enb, NULL, NULL, S1AP_Cause_PR_protocol,
                     S1AP_CauseProtocol_abstract_syntax_error_falsely_constructed_message));
         }
 
@@ -297,7 +300,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             } else {
                 /* Here, if the MME_UE Context is found,
                  * the integrity check is not performed
-                 * For example, ATTACH_REQUEST, 
+                 * For example, ATTACH_REQUEST,
                  * TRACKING_AREA_UPDATE_REQUEST message
                  *
                  * Now, We will check the MAC in the NAS message*/
@@ -305,7 +308,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
                 h.type = e->nas_type;
                 if (h.integrity_protected) {
                     /* Decryption was performed in S1AP handler.
-                     * So, we disabled 'ciphered' 
+                     * So, we disabled 'ciphered'
                      * not to decrypt NAS message */
                     h.ciphered = 0;
                     if (nas_eps_security_decode(mme_ue, h, pkbuf) != OGS_OK) {
@@ -336,7 +339,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
                         S1AP_Cause_PR_nas, S1AP_CauseNas_normal_release,
                         S1AP_UE_CTX_REL_S1_CONTEXT_REMOVE, 0));
             }
-            mme_ue_associate_enb_ue(mme_ue, enb_ue);
+            enb_ue_associate_mme_ue(enb_ue, mme_ue);
         }
 
         ogs_assert(mme_ue);
@@ -372,9 +375,9 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             break;
         }
 
-        bearer = mme_bearer_find_or_add_by_message(mme_ue, &nas_message);
+        bearer = mme_bearer_find_or_add_by_message(
+                    mme_ue, &nas_message, e->create_action);
         if (!bearer) {
-            ogs_error("mme_bearer_find_or_add_by_message() failed");
             ogs_pkbuf_free(pkbuf);
             break;
         }
@@ -405,7 +408,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
 
         } else if (OGS_FSM_CHECK(&bearer->sm, esm_state_exception)) {
 
-            /* 
+            /*
              * The UE requested the wrong APN.
              *
              * From the Issues #568, MME need to accept further service request.
@@ -430,9 +433,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
     case MME_EVT_S6A_MESSAGE:
         mme_ue = e->mme_ue;
         ogs_assert(mme_ue);
-        s6abuf = e->pkbuf;
-        ogs_assert(s6abuf);
-        s6a_message = (ogs_diam_s6a_message_t *)s6abuf->data;
+        s6a_message = e->s6a_message;
         ogs_assert(s6a_message);
 
         enb_ue = enb_ue_cycle(mme_ue->enb_ue);
@@ -441,7 +442,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
 
             ogs_subscription_data_free(
                     &s6a_message->ula_message.subscription_data);
-            ogs_pkbuf_free(s6abuf);
+            ogs_free(s6a_message);
             break;
         }
 
@@ -467,7 +468,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
 
             ogs_subscription_data_free(
                     &s6a_message->ula_message.subscription_data);
-            ogs_pkbuf_free(s6abuf);
+            ogs_free(s6a_message);
             break;
         }
 
@@ -507,15 +508,24 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             break;
         }
         ogs_subscription_data_free(&s6a_message->ula_message.subscription_data);
-        ogs_pkbuf_free(s6abuf);
+        ogs_free(s6a_message);
         break;
 
     case MME_EVT_S11_MESSAGE:
         pkbuf = e->pkbuf;
         ogs_assert(pkbuf);
 
-        if (ogs_gtp_parse_msg(&gtp_message, pkbuf) != OGS_OK) {
-            ogs_error("ogs_gtp_parse_msg() failed");
+        if (ogs_gtp2_parse_msg(&gtp_message, pkbuf) != OGS_OK) {
+            ogs_error("ogs_gtp2_parse_msg() failed");
+            ogs_pkbuf_free(pkbuf);
+            break;
+        }
+
+        gnode = e->gnode;
+        ogs_assert(gnode);
+
+        rv = ogs_gtp_xact_receive(gnode, &gtp_message.h, &xact);
+        if (rv != OGS_OK) {
             ogs_pkbuf_free(pkbuf);
             break;
         }
@@ -554,79 +564,56 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             mme_ue = mme_ue_find_by_teid(gtp_message.h.teid);
         }
 
-        if (mme_ue) {
-            gnode = mme_ue->gnode;
-            ogs_assert(gnode);
-
-        } else {
-            gnode = e->gnode;
-            ogs_assert(gnode);
-        }
-
-        rv = ogs_gtp_xact_receive(gnode, &gtp_message.h, &xact);
-        if (rv != OGS_OK) {
-            ogs_pkbuf_free(pkbuf);
-            break;
-        }
-
         switch (gtp_message.h.type) {
-        case OGS_GTP_ECHO_REQUEST_TYPE:
+        case OGS_GTP2_ECHO_REQUEST_TYPE:
             mme_s11_handle_echo_request(xact, &gtp_message.echo_request);
             break;
-        case OGS_GTP_ECHO_RESPONSE_TYPE:
+        case OGS_GTP2_ECHO_RESPONSE_TYPE:
             mme_s11_handle_echo_response(xact, &gtp_message.echo_response);
             break;
-        case OGS_GTP_CREATE_SESSION_RESPONSE_TYPE:
+        case OGS_GTP2_CREATE_SESSION_RESPONSE_TYPE:
             mme_s11_handle_create_session_response(
                 xact, mme_ue, &gtp_message.create_session_response);
             break;
-        case OGS_GTP_MODIFY_BEARER_RESPONSE_TYPE:
+        case OGS_GTP2_MODIFY_BEARER_RESPONSE_TYPE:
             mme_s11_handle_modify_bearer_response(
                 xact, mme_ue, &gtp_message.modify_bearer_response);
             break;
-        case OGS_GTP_DELETE_SESSION_RESPONSE_TYPE:
+        case OGS_GTP2_DELETE_SESSION_RESPONSE_TYPE:
             mme_s11_handle_delete_session_response(
                 xact, mme_ue, &gtp_message.delete_session_response);
             break;
-        case OGS_GTP_CREATE_BEARER_REQUEST_TYPE:
+        case OGS_GTP2_CREATE_BEARER_REQUEST_TYPE:
             mme_s11_handle_create_bearer_request(
                 xact, mme_ue, &gtp_message.create_bearer_request);
             break;
-        case OGS_GTP_UPDATE_BEARER_REQUEST_TYPE:
+        case OGS_GTP2_UPDATE_BEARER_REQUEST_TYPE:
             mme_s11_handle_update_bearer_request(
                 xact, mme_ue, &gtp_message.update_bearer_request);
             break;
-        case OGS_GTP_DELETE_BEARER_REQUEST_TYPE:
+        case OGS_GTP2_DELETE_BEARER_REQUEST_TYPE:
             mme_s11_handle_delete_bearer_request(
                 xact, mme_ue, &gtp_message.delete_bearer_request);
             break;
-        case OGS_GTP_RELEASE_ACCESS_BEARERS_RESPONSE_TYPE:
+        case OGS_GTP2_RELEASE_ACCESS_BEARERS_RESPONSE_TYPE:
             mme_s11_handle_release_access_bearers_response(
                 xact, mme_ue, &gtp_message.release_access_bearers_response);
             break;
-        case OGS_GTP_DOWNLINK_DATA_NOTIFICATION_TYPE:
-            if (!mme_ue) {
-                if (gtp_message.h.teid_presence)
-                    ogs_warn("No Context : TEID[%d]", gtp_message.h.teid);
-                else
-                    ogs_warn("No Context : No TEID");
-
-                break;
-            }
+        case OGS_GTP2_DOWNLINK_DATA_NOTIFICATION_TYPE:
             mme_s11_handle_downlink_data_notification(
                 xact, mme_ue, &gtp_message.downlink_data_notification);
             break;
-        case OGS_GTP_CREATE_INDIRECT_DATA_FORWARDING_TUNNEL_RESPONSE_TYPE:
+        case OGS_GTP2_CREATE_INDIRECT_DATA_FORWARDING_TUNNEL_RESPONSE_TYPE:
             mme_s11_handle_create_indirect_data_forwarding_tunnel_response(
                 xact, mme_ue,
                 &gtp_message.create_indirect_data_forwarding_tunnel_response);
             break;
-        case OGS_GTP_DELETE_INDIRECT_DATA_FORWARDING_TUNNEL_RESPONSE_TYPE:
+        case OGS_GTP2_DELETE_INDIRECT_DATA_FORWARDING_TUNNEL_RESPONSE_TYPE:
             mme_s11_handle_delete_indirect_data_forwarding_tunnel_response(
                 xact, mme_ue,
                 &gtp_message.delete_indirect_data_forwarding_tunnel_response);
             break;
-        case OGS_GTP_BEARER_RESOURCE_FAILURE_INDICATION_TYPE:
+        case OGS_GTP2_BEARER_RESOURCE_FAILURE_INDICATION_TYPE:
             mme_s11_handle_bearer_resource_failure_indication(
                 xact, mme_ue,
                 &gtp_message.bearer_resource_failure_indication);
@@ -638,6 +625,37 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
         ogs_pkbuf_free(pkbuf);
         break;
 
+    case MME_EVT_S11_TIMER:
+        sgw_ue = e->sgw_ue;
+        ogs_assert(sgw_ue);
+        mme_ue = sgw_ue->mme_ue;
+        ogs_assert(mme_ue);
+
+        switch (e->timer_id) {
+        case MME_TIMER_S11_HOLDING:
+
+            GTP_COUNTER_CLEAR(mme_ue,
+                    GTP_COUNTER_DELETE_SESSION_BY_PATH_SWITCH);
+
+            ogs_list_for_each(&mme_ue->sess_list, sess) {
+
+                GTP_COUNTER_INCREMENT(
+                    mme_ue, GTP_COUNTER_DELETE_SESSION_BY_PATH_SWITCH);
+
+                ogs_assert(OGS_OK ==
+                    mme_gtp_send_delete_session_request(
+                        sgw_ue, sess,
+                        OGS_GTP_DELETE_IN_PATH_SWITCH_REQUEST));
+            }
+            break;
+
+        default:
+            ogs_error("Unknown timer[%s:%d]",
+                    mme_timer_get_name(e->timer_id), e->timer_id);
+        }
+        break;
+
+
     case MME_EVT_SGSAP_LO_SCTP_COMM_UP:
         sock = e->sock;
         ogs_assert(sock);
@@ -658,7 +676,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
         vlr->max_num_of_ostreams =
                 ogs_min(max_num_of_ostreams, vlr->max_num_of_ostreams);
 
-        ogs_debug("VLR-SGs SCTP_COMM_UP[%s] Max Num of Outbound Streams[%d]", 
+        ogs_debug("VLR-SGs SCTP_COMM_UP[%s] Max Num of Outbound Streams[%d]",
             OGS_ADDR(vlr->addr, buf), vlr->max_num_of_ostreams);
 
         e->vlr = vlr;
@@ -684,7 +702,7 @@ void mme_state_operational(ogs_fsm_t *s, mme_event_t *e)
             e->vlr = vlr;
             ogs_fsm_dispatch(&vlr->sm, e);
 
-            ogs_info("VLR-SGs[%s] connection refused!!!", 
+            ogs_info("VLR-SGs[%s] connection refused!!!",
                     OGS_ADDR(vlr->addr, buf));
 
         } else {
