@@ -70,22 +70,22 @@ void pcf_sm_state_operational(ogs_fsm_t *s, pcf_event_t *e)
         SWITCH(message->h.service.name)
         CASE(OGS_SBI_SERVICE_NAME_NPCF_SMPOLICYCONTROL)
             if (!message->h.resource.component[1]) {
-                handled = pcf_npcf_smpolicycontrtol_handle_create(
+                handled = pcf_npcf_smpolicycontrol_handle_create(
                         sess, stream, message);
                 if (!handled) {
                     ogs_error("[%s:%d] "
-                            "pcf_npcf_smpolicycontrtol_handle_create() failed",
+                            "pcf_npcf_smpolicycontrol_handle_create() failed",
                             pcf_ue->supi, sess->psi);
                     OGS_FSM_TRAN(s, pcf_sm_state_exception);
                 }
             } else {
                 SWITCH(message->h.resource.component[2])
                 CASE(OGS_SBI_RESOURCE_NAME_DELETE)
-                    handled = pcf_npcf_smpolicycontrtol_handle_delete(
+                    handled = pcf_npcf_smpolicycontrol_handle_delete(
                             sess, stream, message);
                     if (!handled) {
                         ogs_error("[%s:%d] "
-                            "pcf_npcf_smpolicycontrtol_handle_delete() failed",
+                            "pcf_npcf_smpolicycontrol_handle_delete() failed",
                             pcf_ue->supi, sess->psi);
                         OGS_FSM_TRAN(s, pcf_sm_state_exception);
                     }
@@ -103,17 +103,46 @@ void pcf_sm_state_operational(ogs_fsm_t *s, pcf_event_t *e)
             break;
 
         CASE(OGS_SBI_SERVICE_NAME_NPCF_POLICYAUTHORIZATION)
-            if (!message->h.resource.component[1]) {
-                handled = pcf_npcf_policyauthorization_handle_create(
-                        sess, stream, message);
+            if (message->h.resource.component[1]) {
+                if (message->h.resource.component[2]) {
+                    SWITCH(message->h.resource.component[2])
+                    CASE(OGS_SBI_RESOURCE_NAME_DELETE)
+                        handled = pcf_npcf_policyauthorization_handle_delete(
+                                sess, e->app, stream, message);
+                        break;
+                    DEFAULT
+                        ogs_error("[%s:%d] Invalid resource name [%s]",
+                                pcf_ue->supi, sess->psi,
+                                message->h.resource.component[2]);
+                        ogs_assert(true ==
+                            ogs_sbi_server_send_error(stream,
+                                OGS_SBI_HTTP_STATUS_FORBIDDEN, message,
+                                "Invalid resource name", message->h.uri));
+                    END
+                } else {
+                    SWITCH(message->h.method)
+                    CASE(OGS_SBI_HTTP_METHOD_PATCH)
+                        handled = pcf_npcf_policyauthorization_handle_update(
+                                sess, e->app, stream, message);
+                        break;
+                    DEFAULT
+                        ogs_error("[%s:%d] Unknown method [%s]",
+                                pcf_ue->supi, sess->psi, message->h.method);
+                        ogs_assert(true ==
+                            ogs_sbi_server_send_error(stream,
+                                OGS_SBI_HTTP_STATUS_FORBIDDEN, message,
+                                "Invalid HTTP method", message->h.uri));
+                    END
+                }
             } else {
-                SWITCH(message->h.resource.component[2])
-                CASE(OGS_SBI_RESOURCE_NAME_DELETE)
-                    ogs_fatal("TODO");
+                SWITCH(message->h.method)
+                CASE(OGS_SBI_HTTP_METHOD_POST)
+                    handled = pcf_npcf_policyauthorization_handle_create(
+                            sess, stream, message);
                     break;
                 DEFAULT
-                    ogs_error("[%s:%d] Invalid HTTP URI [%s]",
-                            pcf_ue->supi, sess->psi, message->h.uri);
+                    ogs_error("[%s:%d] Unknown method [%s]",
+                            pcf_ue->supi, sess->psi, message->h.method);
                     ogs_assert(true ==
                         ogs_sbi_server_send_error(stream,
                             OGS_SBI_HTTP_STATUS_FORBIDDEN, message,
