@@ -24,6 +24,7 @@
 
 #include "ogs-gtp.h"
 #include "ogs-diameter-gx.h"
+#include "ogs-diameter-gy.h"
 #include "ogs-diameter-rx.h"
 #include "ogs-diameter-s6b.h"
 #include "ogs-pfcp.h"
@@ -50,7 +51,20 @@ extern int __gsm_log_domain;
 #undef OGS_LOG_DOMAIN
 #define OGS_LOG_DOMAIN __smf_log_domain
 
+typedef enum {
+    SMF_CTF_ENABLED_AUTO = 0,
+    SMF_CTF_ENABLED_YES,
+    SMF_CTF_ENABLED_NO,
+} smf_ctf_enabled_mode;
+
+typedef struct smf_ctf_config_s {
+    smf_ctf_enabled_mode enabled;
+} smf_ctf_config_t;
+
+int smf_ctf_config_init(smf_ctf_config_t *ctf_config);
+
 typedef struct smf_context_s {
+    smf_ctf_config_t    ctf_config;
     const char*         diam_conf_path;   /* SMF Diameter conf path */
     ogs_diam_config_t   *diam_config;     /* SMF Diameter config */
 
@@ -95,6 +109,16 @@ typedef struct smf_ue_s {
     int imsi_len;
     char imsi_bcd[OGS_MAX_IMSI_BCD_LEN+1];
 
+    /* MSISDN */
+    uint8_t msisdn[OGS_MAX_MSISDN_LEN];
+    int msisdn_len;
+    char msisdn_bcd[OGS_MAX_MSISDN_BCD_LEN+1];
+
+    /* IMEI */
+    uint8_t imeisv[OGS_MAX_IMEISV_LEN];
+    int imeisv_len;
+    char  imeisv_bcd[OGS_MAX_IMEISV_BCD_LEN+1];
+
     ogs_list_t sess_list;
 } smf_ue_t;
 
@@ -150,7 +174,7 @@ ED3(uint8_t spare:2;,
 typedef struct smf_bearer_s {
     ogs_lnode_t     lnode;          /**< A node of list_t */
 
-    ogs_lnode_t     to_update_node;
+    ogs_lnode_t     to_modify_node;
     ogs_lnode_t     to_delete_node;
 
     uint32_t        index;
@@ -196,6 +220,20 @@ typedef struct smf_sess_s {
     ogs_sbi_object_t sbi;
     uint32_t        index;          /**< An index of this node */
     ogs_fsm_t       sm;             /* A state machine */
+    struct {
+        bool gx_ccr_init_in_flight; /* Waiting for Gx CCA */
+        uint32_t gx_cca_init_err; /* Gx CCA RXed error code */
+        bool gy_ccr_init_in_flight; /* Waiting for Gy CCA */
+        uint32_t gy_cca_init_err; /* Gy CCA RXed error code */
+        bool gx_ccr_term_in_flight; /* Waiting for Gx CCA */
+        uint32_t gx_cca_term_err; /* Gx CCA RXed error code */
+        bool gy_ccr_term_in_flight; /* Waiting for Gy CCA */
+        uint32_t gy_cca_term_err; /* Gy CCA RXed error code */
+        bool s6b_str_in_flight; /* Waiting for S6B CCA */
+        uint32_t s6b_sta_err; /* S6B CCA RXed error code */
+    } sm_data;
+
+    bool            epc;            /**< EPC or 5GC */
 
     ogs_pfcp_sess_t pfcp;           /* PFCP session context */
 
@@ -217,6 +255,7 @@ typedef struct smf_sess_s {
     ogs_ip_t        gnb_n3_ip;      /* gNB-N3 IPv4/IPv6 */
 
     char            *gx_sid;        /* Gx Session ID */
+    char            *gy_sid;        /* Gx Session ID */
     char            *s6b_sid;       /* S6b Session ID */
 
     OGS_POOL(pf_precedence_pool, uint8_t);
@@ -269,7 +308,7 @@ typedef struct smf_sess_s {
 
     /* PDN Configuration */
     ogs_session_t session;
-    uint8_t ue_pdu_session_type;
+    uint8_t ue_session_type;
     uint8_t ue_ssc_mode;
 
     ogs_pfcp_ue_ip_t *ipv4;
@@ -280,12 +319,30 @@ typedef struct smf_sess_s {
     OpenAPI_rat_type_e sbi_rat_type;
 
     struct {
+        uint8_t version; /* GTPC version */
         ogs_tlv_octet_t ue_pco;
         ogs_tlv_octet_t user_location_information;
         ogs_tlv_octet_t ue_timezone;
+        ogs_tlv_octet_t charging_characteristics;
         bool create_session_response_apn_ambr;
         bool create_session_response_bearer_qos;
-    } gtp; /* Saved from S5-C */
+        uint8_t selection_mode; /* OGS_GTP{1,2}_SELECTION_MODE_*, same in GTPv1C and 2C. */
+        struct {
+            uint8_t nsapi;
+        } v1;  /* GTPv1C specific fields */
+    } gtp; /* Saved from S5-C/Gn */
+
+    struct {
+        uint64_t ul_octets;
+        uint64_t dl_octets;
+        ogs_time_t duration;
+        /* Snapshot of measurement when last report was sent: */
+        struct {
+            uint64_t ul_octets;
+            uint64_t dl_octets;
+            ogs_time_t duration;
+        } last_report;
+    } gy;
 
     struct {
         ogs_nas_extended_protocol_configuration_options_t ue_pco;
@@ -302,9 +359,6 @@ typedef struct smf_sess_s {
         char *n1n2message_location;
     } paging;
 
-    /* Release Holding timer of SMF session context */
-    ogs_timer_t     *t_release_holding;
-
     /* State */
 #define SMF_NGAP_STATE_NONE                                     0
 #define SMF_NGAP_STATE_DELETE_TRIGGER_UE_REQUESTED              1
@@ -345,6 +399,9 @@ typedef struct smf_sess_s {
 
     ogs_list_t      bearer_list;
 
+    ogs_list_t      pdr_to_modify_list;
+    ogs_list_t      qos_flow_to_modify_list;
+
     ogs_gtp_node_t  *gnode;
     ogs_pfcp_node_t *pfcp_node;
 
@@ -357,6 +414,8 @@ smf_context_t *smf_self(void);
 
 int smf_context_parse_config(void);
 
+int smf_use_gy_iface(void);
+
 smf_ue_t *smf_ue_add_by_supi(char *supi);
 smf_ue_t *smf_ue_add_by_imsi(uint8_t *imsi, int imsi_len);
 void smf_ue_remove(smf_ue_t *smf_ue);
@@ -364,7 +423,8 @@ void smf_ue_remove_all(void);
 smf_ue_t *smf_ue_find_by_supi(char *supi);
 smf_ue_t *smf_ue_find_by_imsi(uint8_t *imsi, int imsi_len);
 
-smf_sess_t *smf_sess_add_by_gtp_message(ogs_gtp_message_t *message);
+smf_sess_t *smf_sess_add_by_gtp1_message(ogs_gtp1_message_t *message);
+smf_sess_t *smf_sess_add_by_gtp2_message(ogs_gtp2_message_t *message);
 smf_sess_t *smf_sess_add_by_apn(smf_ue_t *smf_ue, char *apn, uint8_t rat_type);
 
 smf_sess_t *smf_sess_add_by_sbi_message(ogs_sbi_message_t *message);
