@@ -21,6 +21,7 @@
 #define SMF_EVENT_H
 
 #include "ogs-core.h"
+#include "ogs-gtp.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -31,6 +32,9 @@ typedef struct ogs_gtp_xact_s ogs_gtp_xact_t;
 typedef struct ogs_pfcp_node_s ogs_pfcp_node_t;
 typedef struct ogs_pfcp_xact_s ogs_pfcp_xact_t;
 typedef struct ogs_pfcp_message_s ogs_pfcp_message_t;
+typedef struct ogs_diam_gx_message_s ogs_diam_gx_message_t;
+typedef struct ogs_diam_gy_message_s ogs_diam_gy_message_t;
+typedef struct ogs_diam_s6b_message_s ogs_diam_s6b_message_t;
 typedef struct smf_sess_s smf_sess_t;
 typedef struct smf_upf_s smf_upf_t;
 typedef struct ogs_sbi_request_s ogs_sbi_request_t;
@@ -45,7 +49,10 @@ typedef enum {
     SMF_EVT_BASE = OGS_FSM_USER_SIG,
 
     SMF_EVT_S5C_MESSAGE,
+    SMF_EVT_S6B_MESSAGE,
+    SMF_EVT_GN_MESSAGE,
     SMF_EVT_GX_MESSAGE,
+    SMF_EVT_GY_MESSAGE,
 
     SMF_EVT_N4_MESSAGE,
     SMF_EVT_N4_TIMER,
@@ -77,6 +84,17 @@ typedef struct smf_event_s {
     ogs_pfcp_xact_t *pfcp_xact;
     ogs_pfcp_message_t *pfcp_message;
 
+    union {
+        ogs_gtp1_message_t *gtp1_message;
+        ogs_gtp2_message_t *gtp2_message;
+    };
+
+    union {
+        ogs_diam_gx_message_t *gx_message;
+        ogs_diam_gy_message_t *gy_message;
+        ogs_diam_s6b_message_t *s6b_message;
+    };
+
     struct {
         ogs_sbi_request_t *request;
         ogs_sbi_response_t *response;
