@@ -20,7 +20,7 @@
 #ifndef SMF_PFCP_PATH_H
 #define SMF_PFCP_PATH_H
 
-#include "ogs-gtp.h"
+#include "n4-build.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -29,27 +29,36 @@ extern "C" {
 int smf_pfcp_open(void);
 void smf_pfcp_close(void);
 
+int smf_pfcp_send_modify_list(
+        smf_sess_t *sess,
+        ogs_pkbuf_t *(*modify_list)(
+            uint8_t type, smf_sess_t *sess, ogs_pfcp_xact_t *xact),
+        ogs_pfcp_xact_t *xact, ogs_time_t duration);
+
 int smf_5gc_pfcp_send_session_establishment_request(
         smf_sess_t *sess, ogs_sbi_stream_t *stream);
-int smf_5gc_pfcp_send_session_modification_request(
+int smf_5gc_pfcp_send_all_pdr_modification_request(
+        smf_sess_t *sess, ogs_sbi_stream_t *stream,
+        uint64_t flags, ogs_time_t duration);
+int smf_5gc_pfcp_send_qos_flow_list_modification_request(
         smf_sess_t *sess, ogs_sbi_stream_t *stream,
         uint64_t flags, ogs_time_t duration);
-int smf_5gc_pfcp_send_qos_flow_modification_request(smf_bearer_t *qos_flow,
-        ogs_sbi_stream_t *stream, uint64_t flags);
 int smf_5gc_pfcp_send_session_deletion_request(
         smf_sess_t *sess, ogs_sbi_stream_t *stream, int trigger);
 
 int smf_epc_pfcp_send_session_establishment_request(
         smf_sess_t *sess, void *gtp_xact);
-int smf_epc_pfcp_send_session_modification_request(
-        smf_sess_t *sess, void *gtp_xact,
+int smf_epc_pfcp_send_all_pdr_modification_request(
+        smf_sess_t *sess, void *gtp_xact, ogs_pkbuf_t *gtpbuf,
         uint64_t flags, uint8_t gtp_pti, uint8_t gtp_cause);
-int smf_epc_pfcp_send_bearer_modification_request(
+int smf_epc_pfcp_send_one_bearer_modification_request(
         smf_bearer_t *bearer, void *gtp_xact,
         uint64_t flags, uint8_t gtp_pti, uint8_t gtp_cause);
 int smf_epc_pfcp_send_session_deletion_request(
         smf_sess_t *sess, void *gtp_xact);
 
+int smf_epc_pfcp_send_deactivation(smf_sess_t *sess, uint8_t gtp_cause);
+
 int smf_pfcp_send_session_report_response(
         ogs_pfcp_xact_t *xact, smf_sess_t *sess, uint8_t cause);
 
