@@ -36,40 +36,46 @@ bool udm_nudm_ueau_handle_get(
     AuthenticationInfoRequest = recvmsg->AuthenticationInfoRequest;
     if (!AuthenticationInfoRequest) {
         ogs_error("[%s] No AuthenticationInfoRequest", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                recvmsg, "No AuthenticationInfoRequest", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                recvmsg, "No AuthenticationInfoRequest", udm_ue->suci));
         return false;
     }
 
     serving_network_name = AuthenticationInfoRequest->serving_network_name;
     if (!AuthenticationInfoRequest) {
         ogs_error("[%s] No servingNetworkName", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                recvmsg, "No servingNetworkName", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                recvmsg, "No servingNetworkName", udm_ue->suci));
         return false;
     }
 
     if (udm_ue->serving_network_name)
         ogs_free(udm_ue->serving_network_name);
     udm_ue->serving_network_name = ogs_strdup(serving_network_name);
+    ogs_assert(udm_ue->serving_network_name);
 
     ausf_instance_id = AuthenticationInfoRequest->ausf_instance_id;
     if (!AuthenticationInfoRequest) {
         ogs_error("[%s] No ausfInstanceId", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                recvmsg, "No ausfInstanceId", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                recvmsg, "No ausfInstanceId", udm_ue->suci));
         return false;
     }
 
     if (udm_ue->ausf_instance_id)
         ogs_free(udm_ue->ausf_instance_id);
     udm_ue->ausf_instance_id = ogs_strdup(ausf_instance_id);
+    ogs_assert(udm_ue->ausf_instance_id);
 
     ResynchronizationInfo = AuthenticationInfoRequest->resynchronization_info;
     if (!ResynchronizationInfo) {
 
-        udm_sbi_discover_and_send(OpenAPI_nf_type_UDR, udm_ue, stream, NULL,
-                    udm_nudr_dr_build_authentication_subscription);
+        ogs_assert(true == udm_sbi_discover_and_send(
+                    OpenAPI_nf_type_UDR, udm_ue, stream, NULL,
+                    udm_nudr_dr_build_authentication_subscription));
 
     } else {
         uint8_t rand[OGS_RAND_LEN];
@@ -80,17 +86,19 @@ bool udm_nudm_ueau_handle_get(
 
         if (!ResynchronizationInfo->rand) {
             ogs_error("[%s] No RAND", udm_ue->suci);
-            ogs_sbi_server_send_error(stream,
+            ogs_assert(true ==
+                ogs_sbi_server_send_error(stream,
                     OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                    recvmsg, "No RAND", udm_ue->suci);
+                    recvmsg, "No RAND", udm_ue->suci));
             return false;
         }
 
         if (!ResynchronizationInfo->auts) {
             ogs_error("[%s] No AUTS", udm_ue->suci);
-            ogs_sbi_server_send_error(stream,
+            ogs_assert(true ==
+                ogs_sbi_server_send_error(stream,
                     OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                    recvmsg, "No AUTS", udm_ue->suci);
+                    recvmsg, "No AUTS", udm_ue->suci));
             return false;
         }
 
@@ -106,9 +114,10 @@ bool udm_nudm_ueau_handle_get(
             ogs_log_hexdump(OGS_LOG_ERROR, udm_ue->rand, sizeof(udm_ue)->rand);
             ogs_log_hexdump(OGS_LOG_ERROR, rand, sizeof(rand));
 
-            ogs_sbi_server_send_error(stream,
+            ogs_assert(true ==
+                ogs_sbi_server_send_error(stream,
                     OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                    recvmsg, "Invalid RAND", udm_ue->suci);
+                    recvmsg, "Invalid RAND", udm_ue->suci));
             return false;
         }
 
@@ -120,9 +129,10 @@ bool udm_nudm_ueau_handle_get(
             ogs_log_hexdump(OGS_LOG_ERROR, mac_s, OGS_MAC_S_LEN);
             ogs_log_hexdump(OGS_LOG_ERROR, auts + OGS_SQN_LEN, OGS_MAC_S_LEN);
             ogs_log_hexdump(OGS_LOG_ERROR, sqn_ms, OGS_SQN_LEN);
-            ogs_sbi_server_send_error(stream,
+            ogs_assert(true ==
+                ogs_sbi_server_send_error(stream,
                     OGS_SBI_HTTP_STATUS_UNAUTHORIZED,
-                    recvmsg, "Re-sync MAC failed", udm_ue->suci);
+                    recvmsg, "Re-sync MAC failed", udm_ue->suci));
             return false;
 
         }
@@ -151,9 +161,10 @@ bool udm_nudm_ueau_handle_get(
 
         ogs_uint64_to_buffer(sqn, OGS_SQN_LEN, udm_ue->sqn);
 
-        udm_sbi_discover_and_send(OpenAPI_nf_type_UDR,
+        ogs_assert(true ==
+            udm_sbi_discover_and_send(OpenAPI_nf_type_UDR,
                 udm_ue, stream, udm_ue->sqn,
-                udm_nudr_dr_build_authentication_subscription);
+                udm_nudr_dr_build_authentication_subscription));
     }
 
     return true;
@@ -171,51 +182,58 @@ bool udm_nudm_ueau_handle_result_confirmation_inform(
     AuthEvent = message->AuthEvent;
     if (!AuthEvent) {
         ogs_error("[%s] No AuthEvent", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No AuthEvent", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No AuthEvent", udm_ue->suci));
         return false;
     }
 
     if (!AuthEvent->nf_instance_id) {
         ogs_error("[%s] No nfInstanceId", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No nfInstanceId", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No nfInstanceId", udm_ue->suci));
         return false;
     }
 
     if (!AuthEvent->success) {
         ogs_error("[%s] No success", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No success", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No success", udm_ue->suci));
         return false;
     }
 
     if (!AuthEvent->time_stamp) {
         ogs_error("[%s] No timeStamp", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No timeStamp", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No timeStamp", udm_ue->suci));
         return false;
     }
 
     if (!AuthEvent->auth_type) {
         ogs_error("[%s] No authType", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No authType", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No authType", udm_ue->suci));
         return false;
     }
 
     if (!AuthEvent->serving_network_name) {
         ogs_error("[%s] No servingNetworkName", udm_ue->suci);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No servingNetworkName", udm_ue->suci);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No servingNetworkName", udm_ue->suci));
         return false;
     }
 
     udm_ue->auth_event = OpenAPI_auth_event_copy(
             udm_ue->auth_event, message->AuthEvent);
 
-    udm_sbi_discover_and_send(OpenAPI_nf_type_UDR, udm_ue, stream, NULL,
-            udm_nudr_dr_build_update_authentication_status);
+    ogs_assert(true ==
+        udm_sbi_discover_and_send(OpenAPI_nf_type_UDR, udm_ue, stream, NULL,
+            udm_nudr_dr_build_update_authentication_status));
 
     return true;
 }
@@ -233,58 +251,66 @@ bool udm_nudm_uecm_handle_registration(
     Amf3GppAccessRegistration = message->Amf3GppAccessRegistration;
     if (!Amf3GppAccessRegistration) {
         ogs_error("[%s] No Amf3GppAccessRegistration", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No Amf3GppAccessRegistration", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No Amf3GppAccessRegistration", udm_ue->supi));
         return false;
     }
 
     if (!Amf3GppAccessRegistration->dereg_callback_uri) {
         ogs_error("[%s] No dregCallbackUri", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No dregCallbackUri", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No dregCallbackUri", udm_ue->supi));
         return false;
     }
 
     Guami = Amf3GppAccessRegistration->guami;
     if (!Guami) {
         ogs_error("[%s] No Guami", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No Guami", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No Guami", udm_ue->supi));
         return false;
     }
 
     if (!Guami->amf_id) {
         ogs_error("[%s] No Guami.AmfId", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No Guami.AmfId", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No Guami.AmfId", udm_ue->supi));
         return false;
     }
 
     if (!Guami->plmn_id) {
         ogs_error("[%s] No PlmnId", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No PlmnId", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No PlmnId", udm_ue->supi));
         return false;
     }
 
     if (!Guami->plmn_id->mnc) {
         ogs_error("[%s] No PlmnId.Mnc", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No PlmnId.Mnc", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No PlmnId.Mnc", udm_ue->supi));
         return false;
     }
 
     if (!Guami->plmn_id->mcc) {
         ogs_error("[%s] No PlmnId.Mcc", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No PlmnId.Mcc", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No PlmnId.Mcc", udm_ue->supi));
         return false;
     }
 
     if (!Amf3GppAccessRegistration->rat_type) {
         ogs_error("[%s] No RatType", udm_ue->supi);
-        ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
-                message, "No RatType", udm_ue->supi);
+        ogs_assert(true ==
+            ogs_sbi_server_send_error(stream, OGS_SBI_HTTP_STATUS_BAD_REQUEST,
+                message, "No RatType", udm_ue->supi));
         return false;
     }
 
@@ -292,6 +318,7 @@ bool udm_nudm_uecm_handle_registration(
         ogs_free(udm_ue->dereg_callback_uri);
     udm_ue->dereg_callback_uri = ogs_strdup(
             Amf3GppAccessRegistration->dereg_callback_uri);
+    ogs_assert(udm_ue->dereg_callback_uri);
 
     ogs_sbi_parse_guami(&udm_ue->guami, Guami);
 
@@ -302,8 +329,9 @@ bool udm_nudm_uecm_handle_registration(
             udm_ue->amf_3gpp_access_registration,
                 message->Amf3GppAccessRegistration);
 
-    udm_sbi_discover_and_send(OpenAPI_nf_type_UDR, udm_ue, stream, NULL,
-            udm_nudr_dr_build_update_amf_context);
+    ogs_assert(true ==
+        udm_sbi_discover_and_send(OpenAPI_nf_type_UDR, udm_ue, stream, NULL,
+            udm_nudr_dr_build_update_amf_context));
 
     return true;
 }
