@@ -793,13 +793,19 @@ bool udr_nudr_dr_handle_subscription_provisioned(
                 dnnConfigurationList;
 
         memset(&sendmsg, 0, sizeof(sendmsg));
-        sendmsg.SessionManagementSubscriptionData =
-            &SessionManagementSubscriptionData;
+        
+        sendmsg.SessionManagementSubscriptionDataList = OpenAPI_list_create();
+        ogs_assert(sendmsg.SessionManagementSubscriptionDataList);
+
+        OpenAPI_list_add(sendmsg.SessionManagementSubscriptionDataList,
+            &SessionManagementSubscriptionData);
 
         response = ogs_sbi_build_response(&sendmsg, OGS_SBI_HTTP_STATUS_OK);
         ogs_assert(response);
         ogs_assert(true == ogs_sbi_server_send_response(stream, response));
 
+        OpenAPI_list_free(sendmsg.SessionManagementSubscriptionDataList);
+
         if (singleNSSAI.sd)
             ogs_free(singleNSSAI.sd);
 
