@@ -29,7 +29,7 @@ extern "C" {
 int udr_sbi_open(void);
 void udr_sbi_close(void);
 
-void udr_nnrf_nfm_send_nf_register(ogs_sbi_nf_instance_t *nf_instance);
+bool udr_nnrf_nfm_send_nf_register(ogs_sbi_nf_instance_t *nf_instance);
 
 #ifdef __cplusplus
 }
