@@ -45,13 +45,40 @@ extern int __upf_log_domain;
 #define OGS_LOG_DOMAIN __upf_log_domain
 
 typedef struct upf_context_s {
-    ogs_hash_t      *sess_hash;     /* hash table (F-SEID) */
+    ogs_hash_t      *seid_hash;     /* hash table (SEID) */
+    ogs_hash_t      *f_seid_hash;   /* hash table (F-SEID) */
     ogs_hash_t      *ipv4_hash;     /* hash table (IPv4 Address) */
     ogs_hash_t      *ipv6_hash;     /* hash table (IPv6 Address) */
 
     ogs_list_t      sess_list;
 } upf_context_t;
 
+/* Accounting: */
+typedef struct upf_sess_urr_acc_s {
+    bool reporting_enabled;
+    ogs_timer_t *t_time_threshold; /* Time threshold expiration handler */
+    uint32_t time_threshold_start; /* When t_time_threshold started */
+    ogs_pfcp_urr_ur_seqn_t report_seqn; /* Next seqn to use when reporting */
+    uint64_t total_octets;
+    uint64_t ul_octets;
+    uint64_t dl_octets;
+    uint64_t total_pkts;
+    uint64_t ul_pkts;
+    uint64_t dl_pkts;
+    ogs_time_t time_of_first_packet;
+    ogs_time_t time_of_last_packet;
+    /* Snapshot of measurement when last report was sent: */
+    struct {
+        uint64_t total_octets;
+        uint64_t ul_octets;
+        uint64_t dl_octets;
+        uint64_t total_pkts;
+        uint64_t ul_pkts;
+        uint64_t dl_pkts;
+        ogs_time_t timestamp;
+    } last_report;
+} upf_sess_urr_acc_t;
+
 #define UPF_SESS(pfcp_sess) ogs_container_of(pfcp_sess, upf_sess_t, pfcp)
 typedef struct upf_sess_s {
     ogs_lnode_t     lnode;
@@ -60,7 +87,10 @@ typedef struct upf_sess_s {
     ogs_pfcp_sess_t pfcp;
 
     uint64_t        upf_n4_seid;        /* UPF SEID is dervied from INDEX */
-    uint64_t        smf_n4_seid;        /* SMF SEID is received from Peer */
+    struct {
+        uint64_t    seid;
+        ogs_ip_t    ip;
+    } smf_n4_f_seid;                    /* SMF SEID is received from Peer */
 
     /* APN Configuration */
     ogs_pfcp_ue_ip_t *ipv4;
@@ -68,6 +98,9 @@ typedef struct upf_sess_s {
 
     char            *gx_sid;            /* Gx Session ID */
     ogs_pfcp_node_t *pfcp_node;
+
+    /* Accounting: */
+    upf_sess_urr_acc_t urr_acc[OGS_MAX_NUM_OF_URR]; /* FIXME: This probably needs to be mved to a hashtable or alike */
 } upf_sess_t;
 
 void upf_context_init(void);
@@ -82,14 +115,21 @@ upf_sess_t *upf_sess_add(ogs_pfcp_f_seid_t *f_seid);
 int upf_sess_remove(upf_sess_t *sess);
 void upf_sess_remove_all(void);
 upf_sess_t *upf_sess_find(uint32_t index);
-upf_sess_t *upf_sess_find_by_cp_seid(uint64_t seid);
-upf_sess_t *upf_sess_find_by_up_seid(uint64_t seid);
+upf_sess_t *upf_sess_find_by_smf_n4_seid(uint64_t seid);
+upf_sess_t *upf_sess_find_by_smf_n4_f_seid(ogs_pfcp_f_seid_t *f_seid);
+upf_sess_t *upf_sess_find_by_upf_n4_seid(uint64_t seid);
 upf_sess_t *upf_sess_find_by_ipv4(uint32_t addr);
 upf_sess_t *upf_sess_find_by_ipv6(uint32_t *addr6);
 
 uint8_t upf_sess_set_ue_ip(upf_sess_t *sess,
         uint8_t session_type, ogs_pfcp_pdr_t *pdr);
 
+void upf_sess_urr_acc_add(upf_sess_t *sess, ogs_pfcp_urr_t *urr, size_t size, bool is_uplink);
+void upf_sess_urr_acc_fill_usage_report(upf_sess_t *sess, const ogs_pfcp_urr_t *urr,
+                                        ogs_pfcp_user_plane_report_t *report, unsigned int idx);
+void upf_sess_urr_acc_snapshot(upf_sess_t *sess, ogs_pfcp_urr_t *urr);
+void upf_sess_urr_acc_time_threshold_setup(upf_sess_t *sess, ogs_pfcp_urr_t *urr);
+
 #ifdef __cplusplus
 }
 #endif
