@@ -26,6 +26,7 @@ abts_suite *test_idle(abts_suite *suite);
 abts_suite *test_emm_status(abts_suite *suite);
 abts_suite *test_ue_context(abts_suite *suite);
 abts_suite *test_reset(abts_suite *suite);
+abts_suite *test_issues(abts_suite *suite);
 
 const struct testlist {
     abts_suite *(*func)(abts_suite *suite);
@@ -37,6 +38,7 @@ const struct testlist {
     {test_emm_status},
     {test_ue_context},
     {test_reset},
+    {test_issues},
     {NULL},
 };
 
